using CSIP_42.Model.Entity;
using CSIP_42.Model.Entity.Games;
using CSIP_42.Model.Enumerators;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace CSIP_42.Model.Construction.Test
{
    [TestClass]
    public class UtilityTests
    {
        [TestMethod]
        public void TestEntityIdentifier()
        {
            Model model = new Model();
            model.NewGame();
            Assert.IsNotNull(model);
            var id1 = model.GetEntityIdentifier(TypeNameEnum.SushiRestaurant);
            var id3 = model.GetEntityIdentifier(TypeNameEnum.SushiRestaurant);
            var id2 = model.GetEntityIdentifier(TypeNameEnum.DodgemGame);
            Assert.IsFalse(id1.Equals(id2));
            Assert.IsFalse(id1==id2);
            Assert.IsTrue(id1!=id2);
            
            Assert.IsTrue(id1.Equals(id3));
            Assert.IsTrue(id1==id3);
            Assert.IsFalse(id1!=id3);
            
            Assert.IsNotNull(id3.GetHashCode());
            Assert.IsNotNull(id2.GetHashCode());
            Assert.IsNotNull(id1.GetHashCode());
            
            Assert.IsFalse(id1.Equals(null));
            Assert.IsFalse(id1.Equals("Test"));
        }

        [TestMethod]
        public void EntitiyIdentifierConstructor()
        {
            var id = new EntityIdentifier(TypeNameEnum.Cleaner, CategoryEnum.Staff, 1000, 1, 1);
            Assert.AreEqual(id, new EntityIdentifier(TypeNameEnum.Cleaner, CategoryEnum.Staff, 1000, 1, 1));
        }
        
        [TestMethod]
        public void NullPriceAndMinCapDisplayInfoTest()
        {
            var id = new EntityIdentifier(TypeNameEnum.Cleaner, CategoryEnum.Staff, 1000, 1, 1);
            var id2 = new EntityIdentifier(TypeNameEnum.Cleaner, CategoryEnum.Staff, 1000, 1, 1); 
            var id3 = new EntityIdentifier(TypeNameEnum.Cleaner, CategoryEnum.Staff, 1000, 1, 1); 


            Assert.IsNull(id.ConstructionInformation);
            id.SetMinimumCapacityUponConstruction(30);
            id.SetTicketPriceUponConstruction(2300);
            
            id2.SetMinimumCapacityUponConstruction(30);
            id2.SetTicketPriceUponConstruction(2300);
            
            id3.SetMinimumCapacityUponConstruction(40);
            id3.SetTicketPriceUponConstruction(2500);
            
            Assert.AreEqual(id.ConstructionInformation.MinimumCapacity,30);
            Assert.AreEqual(id.ConstructionInformation.TicketPrice,2300);
            
            Assert.IsFalse(id.ConstructionInformation.Equals(null));
            Assert.IsFalse(id.ConstructionInformation.Equals(id3.ConstructionInformation));
            Assert.IsTrue(id.ConstructionInformation != id3.ConstructionInformation);
            Assert.IsTrue(id.ConstructionInformation.Equals(id2.ConstructionInformation));
            
            Assert.IsNotNull(id.GetHashCode());
            Assert.IsNotNull(id2.GetHashCode());
            Assert.IsNotNull(id3.GetHashCode());
            
            Assert.AreNotEqual(id.ConstructionInformation,null);

            id.DisplayInformationEnglish = new DisplayInformation();
            id2.DisplayInformationEnglish = new DisplayInformation();
            id3.DisplayInformationEnglish = new DisplayInformation();
            
            Assert.IsTrue(id.DisplayInformationEnglish == id2.DisplayInformationEnglish);
            Assert.IsTrue(id.DisplayInformationEnglish.Equals(id2.DisplayInformationEnglish));
            Assert.IsFalse(id.DisplayInformationEnglish.Equals(null));
            Assert.IsFalse(id.DisplayInformationEnglish.Equals("Tesz"));

            id3.DisplayInformationEnglish.DisplayName = "valami_mas";
            Assert.IsFalse(id.DisplayInformationEnglish.Equals(id3.DisplayInformationEnglish));
            Assert.IsTrue(id.DisplayInformationEnglish != id3.DisplayInformationEnglish);

        }
        
        [TestMethod]
        public void EntityInfoTest()
        {
            var id = new EntityInfo(10, 5, 20, 3, 1000, 200, 1000, CategoryEnum.Game);

            Assert.AreEqual(id.Category, CategoryEnum.Game);
            Assert.AreEqual(id.SingleCost, 1000);
            Assert.AreEqual(id.AlwaysCost, 200);
            Assert.AreEqual(id.TicketPrice, 1000);
            Assert.AreEqual(id.MinimumCapacity,3 );
            Assert.AreEqual(id.MaximumCapacity, 20);
            Assert.AreEqual(id.PeopleUsing, 5);
            Assert.AreEqual(id.PeopleWaiting, 10);

            
            
        }
        
        
    }
}