using Microsoft.VisualStudio.TestTools.UnitTesting;
using CSIP_42.Model;
using CSIP_42.Model.Enumerators;
using CSIP_42.Model.Entity;
using System.Collections.Generic;
using System;
using Satsuma;



namespace CSIP_42.ThemePark.Model.Simulation.Test
{
    [TestClass]
    public class SimulationTest
    {

        #region GUESTE TEST
        [TestMethod]
        public void GuestTest_Construktor()
        {
            Guest.MinCash = 8000;
            Guest.MaxCash = 35000;
            Guest.MinHappy = 500;
            Guest.MaxHappy = 1000;
            Guest.MinHunger = 500;
            Guest.MaxHunger = 1000;
            Guest.MinNaturesCall = 500;
            Guest.MaxNaturesCall = 1000;
            Guest.MinWillingness = 1;
            Guest.MaxWillingness = 3;
            CSIP_42.Model.Model model = new CSIP_42.Model.Model(true, "EntityConfigForSimulationTest.json");
            Guest guest = new Guest(model.EntityIdentifiers[TypeNameEnum.Guest], model.GetGameInfo().Table, model.GetMainGateGatePos());
            Assert.IsTrue(Guest.MinCash <= guest.Cash && guest.Cash <= Guest.MaxCash);
            Assert.IsTrue(Guest.MinHappy <= guest.Happiness && guest.Happiness <= Guest.MaxHappy);
            Assert.IsTrue(Guest.MinHunger <= guest.Hunger && guest.Hunger <= Guest.MaxHunger);
            Assert.IsTrue(Guest.MinNaturesCall <= guest.NaturesCall && guest.NaturesCall <= Guest.MaxNaturesCall);
            Assert.IsTrue(Guest.MinWillingness <= guest.willingness && guest.willingness <= Guest.MaxWillingness);
        }

        [TestMethod]
        public void GuestTest_maxPrice()
        {
            Guest.MinCash = 8000;
            Guest.MaxCash = 35000;
            Guest.MinHappy = 500;
            Guest.MaxHappy = 1000;
            Guest.MinHunger = 500;
            Guest.MaxHunger = 1000;
            Guest.MinNaturesCall = 500;
            Guest.MaxNaturesCall = 1000;
            Guest.MinWillingness = 1;
            Guest.MaxWillingness = 3;
            CSIP_42.Model.Model model = new CSIP_42.Model.Model(true, "EntityConfigForSimulationTest.json");
            Guest guest = new Guest(model.EntityIdentifiers[TypeNameEnum.Guest], model.GetGameInfo().Table, model.GetMainGateGatePos());
            guest.Cash = 10000;
            guest.willingness = 1;
            Assert.AreEqual(guest.MaxPrice(), 4000);
            guest.willingness = 2;
            Assert.AreEqual(guest.MaxPrice(), 6000);
            guest.willingness = 3;
            Assert.AreEqual(guest.MaxPrice(), 8000);
            guest.Cash = 1000;
            guest.willingness = 1;
            Assert.AreEqual(guest.MaxPrice(), 500);
            guest.willingness = 2;
            Assert.AreEqual(guest.MaxPrice(), 700);
            guest.willingness = 3;
            Assert.AreEqual(guest.MaxPrice(), 1000);
            guest.Cash = 100;
            guest.willingness = 1;
            Assert.AreEqual(guest.MaxPrice(), 100);
            guest.willingness = 2;
            Assert.AreEqual(guest.MaxPrice(), 100);
            guest.willingness = 3;
            Assert.AreEqual(guest.MaxPrice(), 100);

        }

        [TestMethod]
        public void GuestTest_status()
        {
            Guest.MinCash = 8000;
            Guest.MaxCash = 35000;
            Guest.MinHappy = 500;
            Guest.MaxHappy = 1000;
            Guest.MinHunger = 500;
            Guest.MaxHunger = 1000;
            Guest.MinNaturesCall = 500;
            Guest.MaxNaturesCall = 1000;
            Guest.MinWillingness = 1;
            Guest.MaxWillingness = 3;
            CSIP_42.Model.Model model = new CSIP_42.Model.Model(true, "EntityConfigForSimulationTest.json");
            Guest guest = new Guest(model.EntityIdentifiers[TypeNameEnum.Guest], model.GetGameInfo().Table, model.GetMainGateGatePos());
            Guest.MinHappy = Guest.Okay_Happy_Limit + 1;
            Guest.MaxHappy = Guest.HappinessMaximumValue;
            Guest.MinHunger = Guest.Okay_Happy_Limit + 1;
            Guest.MaxHunger = Guest.HappinessMaximumValue;
            for (int i = 0; i < 10; i++)
            {
                guest = new Guest(model.EntityIdentifiers[TypeNameEnum.Guest], model.GetGameInfo().Table, model.GetMainGateGatePos());
                Assert.AreEqual(guest.status, StatusEnum.Happy);
            }
            Guest.MinHappy = Guest.Sad_Okay_Limit + 1;
            Guest.MaxHappy = Guest.Okay_Happy_Limit - 1;
            Guest.MinHunger = Guest.Sad_Okay_Limit + 1;
            Guest.MaxHunger = Guest.Okay_Happy_Limit - 1;
            for (int i = 0; i < 10; i++)
            {
                guest = new Guest(model.EntityIdentifiers[TypeNameEnum.Guest], model.GetGameInfo().Table, model.GetMainGateGatePos());
                Assert.AreEqual(guest.status, StatusEnum.Okay);
            }
            Guest.MinHappy = 1;
            Guest.MaxHappy = Guest.Sad_Okay_Limit - 1;
            Guest.MinHunger = 1;
            Guest.MaxHunger = Guest.Sad_Okay_Limit - 1;
            for (int i = 0; i < 10; i++)
            {
                guest = new Guest(model.EntityIdentifiers[TypeNameEnum.Guest], model.GetGameInfo().Table, model.GetMainGateGatePos());
                Assert.AreEqual(guest.status, StatusEnum.Sad);
            }

        }

        [TestMethod]
        public void GuestTest_wannaLeave()
        {
            Guest.MinCash = 8000;
            Guest.MaxCash = 35000;
            Guest.MinHappy = 500;
            Guest.MaxHappy = 1000;
            Guest.MinHunger = 500;
            Guest.MaxHunger = 1000;
            Guest.MinNaturesCall = 500;
            Guest.MaxNaturesCall = 1000;
            Guest.MinWillingness = 1;
            Guest.MaxWillingness = 3;
            CSIP_42.Model.Model model = new CSIP_42.Model.Model(true, "EntityConfigForSimulationTest.json");
            Guest guest = new Guest(model.EntityIdentifiers[TypeNameEnum.Guest], model.GetGameInfo().Table, model.GetMainGateGatePos());
            Guest.MinHappy = 1;
            Guest.MaxHappy = 100;
            Guest.MinHunger = 1;
            Guest.MaxHunger = 100;
            Guest.MinNaturesCall = 1;
            Guest.MaxNaturesCall = 100;
            guest = new Guest(model.EntityIdentifiers[TypeNameEnum.Guest], model.GetGameInfo().Table, model.GetMainGateGatePos());
            Assert.IsFalse(guest.wannaLeave);
            Assert.AreNotEqual(guest.WhatWant(), CategoryEnum.MainGate);

            Guest.MinHappy = -10;
            Guest.MaxHappy = 0;
            Guest.MinHunger = 1;
            Guest.MaxHunger = 100;
            Guest.MinNaturesCall = 1;
            Guest.MaxNaturesCall = 100;
            guest = new Guest(model.EntityIdentifiers[TypeNameEnum.Guest], model.GetGameInfo().Table, model.GetMainGateGatePos());
            Assert.IsTrue(guest.wannaLeave);
            Assert.AreEqual(guest.status, StatusEnum.Sad);
            Assert.AreEqual(guest.WhatWant(), CategoryEnum.MainGate);

            Guest.MinHappy = Guest.Okay_Happy_Limit + 1;
            Guest.MaxHappy = Guest.HappinessMaximumValue;
            Guest.MinHunger = -10;
            Guest.MaxHunger = 0;
            Guest.MinNaturesCall = 1;
            Guest.MaxNaturesCall = 100;
            guest = new Guest(model.EntityIdentifiers[TypeNameEnum.Guest], model.GetGameInfo().Table, model.GetMainGateGatePos());
            Assert.IsTrue(guest.wannaLeave);
            Assert.AreEqual(guest.status, StatusEnum.Sad);
            Assert.AreEqual(guest.WhatWant(), CategoryEnum.MainGate);

            Guest.MinHappy = Guest.Okay_Happy_Limit + 1;
            Guest.MaxHappy = Guest.HappinessMaximumValue;
            Guest.MinHunger = 1;
            Guest.MaxHunger = 100;
            Guest.MinNaturesCall = -10;
            Guest.MaxNaturesCall = 0;
            guest = new Guest(model.EntityIdentifiers[TypeNameEnum.Guest], model.GetGameInfo().Table, model.GetMainGateGatePos());
            Assert.IsTrue(guest.wannaLeave);
            Assert.AreEqual(guest.status, StatusEnum.Sad);
            Assert.AreEqual(guest.WhatWant(), CategoryEnum.MainGate);

        }

        [TestMethod]
        public void GuestTest_WhatWant()
        {
            Guest.MinCash = 8000;
            Guest.MaxCash = 35000;
            Guest.MinHappy = 500;
            Guest.MaxHappy = 1000;
            Guest.MinHunger = 500;
            Guest.MaxHunger = 1000;
            Guest.MinNaturesCall = 500;
            Guest.MaxNaturesCall = 1000;
            Guest.MinWillingness = 1;
            Guest.MaxWillingness = 3;
            CSIP_42.Model.Model model = new CSIP_42.Model.Model(true, "EntityConfigForSimulationTest.json");
            Guest guest = new Guest(model.EntityIdentifiers[TypeNameEnum.Guest], model.GetGameInfo().Table, model.GetMainGateGatePos());
            Guest.MinWillingness = 1;
            Guest.MaxWillingness = 3;
            
            for (int i = 0; i < 10; i++)
            {
                guest = new Guest(model.EntityIdentifiers[TypeNameEnum.Guest], model.GetGameInfo().Table, model.GetMainGateGatePos());
                guest.wannaLeave = true;
                Assert.AreEqual(guest.WhatWant(), CategoryEnum.MainGate);
            }

            Guest.MinHappy = 500;
            Guest.MaxHappy = 1000;
            Guest.MinHunger =  Guest.MaxHunger = Guest.HungerMaximumValue + 1;
            Guest.MinNaturesCall =  Guest.MaxNaturesCall = Guest.NaturesCallMaximumValue + 1;
            for (int i = 0; i < 10; i++)
            {
                guest = new Guest(model.EntityIdentifiers[TypeNameEnum.Guest], model.GetGameInfo().Table, model.GetMainGateGatePos());
                Assert.AreEqual(guest.WhatWant(), CategoryEnum.Game);
            }

            Guest.MinHappy = Guest.MaxHappy = Guest.HappinessMaximumValue + 1;
            Guest.MinHunger = 500;
            Guest.MaxHunger = 1000;
            Guest.MinNaturesCall = Guest.MaxNaturesCall = Guest.NaturesCallMaximumValue + 1;
            for (int i = 0; i < 10; i++)
            {
                guest = new Guest(model.EntityIdentifiers[TypeNameEnum.Guest], model.GetGameInfo().Table, model.GetMainGateGatePos());
                Assert.AreEqual(guest.WhatWant(), CategoryEnum.Restaurant);
            }

            Guest.MinHappy = Guest.MaxHappy = Guest.HappinessMaximumValue + 1;
            Guest.MinHunger = Guest.MaxHunger = Guest.HungerMaximumValue + 1;
            Guest.MinNaturesCall = 500;
            Guest.MaxNaturesCall = 1000;
            for (int i = 0; i < 10; i++)
            {
                guest = new Guest(model.EntityIdentifiers[TypeNameEnum.Guest], model.GetGameInfo().Table, model.GetMainGateGatePos());
                Assert.AreEqual(guest.WhatWant(), CategoryEnum.Toalett);
            }


        }

        [TestMethod]
        public void GuestTest_waiting()
        {
            Guest.MinCash = 8000;
            Guest.MaxCash = 35000;
            Guest.MinHappy = 500;
            Guest.MaxHappy = 1000;
            Guest.MinHunger = 500;
            Guest.MaxHunger = 1000;
            Guest.MinNaturesCall = 500;
            Guest.MaxNaturesCall = 1000;
            Guest.MinWillingness = 1;
            Guest.MaxWillingness = 3;
            Guest.SecHappyMinus = 0;
              CSIP_42.Model.Model model = new CSIP_42.Model.Model(true, "EntityConfigForSimulationTest.json");
            
            Guest guest = new Guest(model.EntityIdentifiers[TypeNameEnum.Guest], model.GetGameInfo().Table, model.GetMainGateGatePos());
            guest.Happiness = 200;
            guest.place = PersonPlaceEnum.InQueue;
            for (int i = 1; i <= 10; i++)
            {
                Assert.IsFalse(guest.Move());
                Assert.AreEqual(guest.Happiness, 200 - i * 10);
            }
        }

        [TestMethod]
        public void GuestTest_enjoy()
        {
            Guest.MinCash = 8000;
            Guest.MaxCash = 35000;
            Guest.MinHappy = 500;
            Guest.MaxHappy = 1000;
            Guest.MinHunger = 500;
            Guest.MaxHunger = 1000;
            Guest.MinNaturesCall = 500;
            Guest.MaxNaturesCall = 1000;
            Guest.MinWillingness = 1;
            Guest.MaxWillingness = 3;
              CSIP_42.Model.Model model = new CSIP_42.Model.Model(true, "EntityConfigForSimulationTest.json");
            
            Guest guest = new Guest(model.EntityIdentifiers[TypeNameEnum.Guest], model.GetGameInfo().Table, model.GetMainGateGatePos());
            guest.ServizeTimeRemaining = 10;
            guest.place = PersonPlaceEnum.InBuilding;
            for (int i = 1; i <= 10; i++)
            {
                guest.Move();
                Assert.AreEqual(guest.ServizeTimeRemaining, 10-i);
            }
        }

        [TestMethod]
        public void GuestTest_step()
        {
            Guest.MinCash = 8000;
            Guest.MaxCash = 35000;
            Guest.MinHappy = 500;
            Guest.MaxHappy = 1000;
            Guest.MinHunger = 500;
            Guest.MaxHunger = 1000;
            Guest.MinNaturesCall = 500;
            Guest.MaxNaturesCall = 1000;
            Guest.MinWillingness = 1;
            Guest.MaxWillingness = 3;
            CSIP_42.Model.Model.N = 11;
            CSIP_42.Model.Model.M = 10;
            CSIP_42.Model.Model.spawnSec= 0;

            CSIP_42.Model.Model model = new CSIP_42.Model.Model(true, "EntityConfigForSimulationTest.json");
            
            Guest guest = new Guest(model.EntityIdentifiers[TypeNameEnum.Guest], model.GetGameInfo().Table, model.GetMainGateGatePos());

            model.SelectForCurrentElement(TypeNameEnum.Road);
            List<Tuple<Position, int>> init = new List<Tuple<Position, int>>();
            init.Add(new Tuple<Position, int>(new Position(6, 6), 3));
            init.Add(new Tuple<Position, int>(new Position(7, 6), 4));
            init.Add(new Tuple<Position, int>(new Position(7, 5), 5));
            init.Add(new Tuple<Position, int>(new Position(7, 4), 6));
            init.Add(new Tuple<Position, int>(new Position(6, 4), 7));
            init.Add(new Tuple<Position, int>(new Position(5, 4), 8));
            init.Add(new Tuple<Position, int>(new Position(6, 3), 9));
            init.Add(new Tuple<Position, int>(new Position(7, 3), 10));

            for (int i = 0; i < 7; i++)
            {
                model.PlaceEntity(init[i].Item1, init[i].Item2);
            }

            model.SelectForCurrentElement(TypeNameEnum.HotDogRestaurant);
            model.SetEntryPriceForCurrentElement(0);
            model.PlaceEntity(new Position(4, 3), 11);

            GameObject gameinfo = model.GetGameInfo();
            gameinfo.People[TypeNameEnum.Guest].Add(guest);

            Dijkstra dijkstra = new Dijkstra(gameinfo.Graph, arc => 1, DijkstraMode.Sum);
            Node  sourceNode = gameinfo.mainGate.Node;

            Node targetNode = (gameinfo.Table[4,3] as Restaurant).Node;
            dijkstra.AddSource(sourceNode);
            dijkstra.RunUntilFixed(targetNode);
            var disst = dijkstra.GetDistance(targetNode);
            Path path = (dijkstra.GetPath(targetNode) as Path);
            guest.SetPath(path);

            Assert.IsFalse(guest.Move());
            Assert.AreEqual(guest.position, new Position(5,6));

            for (int i = 0; i < 6; i++)
            {
                Assert.IsFalse(guest.Move());
                Assert.AreEqual(guest.position, init[i].Item1);
            }
            Assert.IsTrue(guest.Move());
            Assert.AreEqual(guest.position, new Position(5,3));

        }

        [TestMethod]
        public void GuestTest_trashing()
        {
            Guest.MinCash = 8000;
            Guest.MaxCash = 35000;
            Guest.MinHappy = 500;
            Guest.MaxHappy = 1000;
            Guest.MinHunger = 500;
            Guest.MaxHunger = 1000;
            Guest.MinNaturesCall = 500;
            Guest.MaxNaturesCall = 1000;
            Guest.MinWillingness = 1;
            Guest.MaxWillingness = 3;
            CSIP_42.Model.Model.N = 11;
            CSIP_42.Model.Model.M = 10;
            CSIP_42.Model.Model.spawnSec = 0;

            CSIP_42.Model.Model model = new CSIP_42.Model.Model(true, "EntityConfigForSimulationTest.json");

            Guest guest = new Guest(model.EntityIdentifiers[TypeNameEnum.Guest], model.GetGameInfo().Table, model.GetMainGateGatePos());
            guest.TrashTimeRemaining = 4;

            model.SelectForCurrentElement(TypeNameEnum.Road);
            List<Tuple<Position, int>> init = new List<Tuple<Position, int>>();
            init.Add(new Tuple<Position, int>(new Position(6, 6), 3));
            init.Add(new Tuple<Position, int>(new Position(7, 6), 4));
            init.Add(new Tuple<Position, int>(new Position(7, 5), 5));
            init.Add(new Tuple<Position, int>(new Position(7, 4), 6));
            init.Add(new Tuple<Position, int>(new Position(6, 4), 7));
            init.Add(new Tuple<Position, int>(new Position(5, 4), 8));
            init.Add(new Tuple<Position, int>(new Position(6, 3), 9));
            init.Add(new Tuple<Position, int>(new Position(7, 3), 10));

            for (int i = 0; i < 7; i++)
            {
                model.PlaceEntity(init[i].Item1, init[i].Item2);
            }

            model.SelectForCurrentElement(TypeNameEnum.HotDogRestaurant);
            model.SetEntryPriceForCurrentElement(0);
            model.PlaceEntity(new Position(4, 3), 11);

            GameObject gameinfo = model.GetGameInfo();
            gameinfo.People[TypeNameEnum.Guest].Add(guest);

            Dijkstra dijkstra = new Dijkstra(gameinfo.Graph, arc => 1, DijkstraMode.Sum);
            Node sourceNode = gameinfo.mainGate.Node;

            Node targetNode = (gameinfo.Table[4, 3] as Restaurant).Node;
            dijkstra.AddSource(sourceNode);
            dijkstra.RunUntilFixed(targetNode);
            var disst = dijkstra.GetDistance(targetNode);
            Path path = (dijkstra.GetPath(targetNode) as Path);
            guest.SetPath(path);

            guest.Move();
            Assert.IsFalse(guest.wannaThrowTrash);
            Assert.AreEqual(guest.position, new Position(5, 6));

            for (int i = 0; i < 2; i++)
            {
                guest.Move();
                Assert.IsFalse(guest.wannaThrowTrash);
                Assert.AreEqual(guest.position, init[i].Item1);
            }

            guest.Move();
            Assert.IsTrue(guest.wannaThrowTrash);
            Assert.AreEqual(guest.position, new Position(7, 5));
            
            for (int i = 3; i < 6; i++) {
                guest.Move();
                Assert.IsFalse(guest.wannaThrowTrash);
                Assert.AreEqual(guest.position, init[i].Item1);
            }

        }


        [TestMethod]
        public void GuestTest_binRecognize()
        {
            Guest.MinCash = 8000;
            Guest.MaxCash = 35000;
            Guest.MinHappy = 500;
            Guest.MaxHappy = 1000;
            Guest.MinHunger = 500;
            Guest.MaxHunger = 1000;
            Guest.MinNaturesCall = 500;
            Guest.MaxNaturesCall = 1000;
            Guest.MinWillingness = 1;
            Guest.MaxWillingness = 3;
            CSIP_42.Model.Model.N = 11;
            CSIP_42.Model.Model.M = 10;
            CSIP_42.Model.Model.spawnSec = 0;

            CSIP_42.Model.Model model = new CSIP_42.Model.Model(true, "EntityConfigForSimulationTest.json");

            Guest guest = new Guest(model.EntityIdentifiers[TypeNameEnum.Guest], model.GetGameInfo().Table, model.GetMainGateGatePos());
            guest.TrashTimeRemaining = 4;

            model.SelectForCurrentElement(TypeNameEnum.Road);
            List<Tuple<Position, int>> init = new List<Tuple<Position, int>>();
            init.Add(new Tuple<Position, int>(new Position(6, 6), 3));
            init.Add(new Tuple<Position, int>(new Position(7, 6), 4));
            init.Add(new Tuple<Position, int>(new Position(7, 5), 5));
            init.Add(new Tuple<Position, int>(new Position(7, 4), 6));
            init.Add(new Tuple<Position, int>(new Position(6, 4), 7));
            init.Add(new Tuple<Position, int>(new Position(5, 4), 8));

            for (int i = 0; i < init.Count; i++)
            {
                model.PlaceEntity(init[i].Item1, init[i].Item2);
            }

            model.SelectForCurrentElement(TypeNameEnum.HotDogRestaurant);
            model.SetEntryPriceForCurrentElement(0);
            model.PlaceEntity(new Position(4, 3), 11);

            model.SelectForCurrentElement(TypeNameEnum.Bin);
            model.PlaceEntity(new Position(8, 6), 12);

            GameObject gameinfo = model.GetGameInfo();
            gameinfo.People[TypeNameEnum.Guest].Add(guest);

            Dijkstra dijkstra = new Dijkstra(gameinfo.Graph, arc => 1, DijkstraMode.Sum);
            Node sourceNode = gameinfo.mainGate.Node;

            Node targetNode = (gameinfo.Table[4, 3] as Restaurant).Node;
            dijkstra.AddSource(sourceNode);
            dijkstra.RunUntilFixed(targetNode);
            var disst = dijkstra.GetDistance(targetNode);
            Path path = (dijkstra.GetPath(targetNode) as Path);
            guest.SetPath(path);

            guest.Move();
            Assert.IsFalse(guest.wannaThrowTrash);
            
            for (int i = 0; i < init.Count; i++)
            {
                guest.Move();
                Assert.IsFalse(guest.wannaThrowTrash);
                Assert.AreEqual(guest.position, init[i].Item1);

            }

        }


        /*
          0 1 2 3 4 5 6 7 8 9 10
          _ _ _ _ _ _ _ _ _ _ _ 
        0|F|F|F|F|F|F|F|F|F|F|F|
        1|F|_|_|_|_|_|_|_|_|_|F|
        2|F|hot|r|r|r|3|r|1|_|F|
        3|F|_|_|_|_|_|_|_|r|_|F|
        4|F|_|_|_|_|_|_|_|r|t|F|
        5|F|_|_|_|_|r|r|r|r|_|F|
        6|F|_|_|_|g|r|_|v|_|_|F|
        7|F|_|_|   |^|   |_|_|F|
        8|F|_|_|         |_|_|F|
        9|F|F|F|_ _ _ _ _|F|F|F|
          
         */
        [TestMethod]
        public void GuestTest_happinessModifiers()
        {
            Guest.MinCash = 8000;
            Guest.MaxCash = 35000;
            Guest.MinHappy = 500;
            Guest.MaxHappy = 700;
            Guest.MinHunger = 500;
            Guest.MaxHunger = 1000;
            Guest.MinNaturesCall = 500;
            Guest.MaxNaturesCall = 1000;
            Guest.MinWillingness = 1;
            Guest.MaxWillingness = 3;
            Guest.SecHappyMinus = 0;
            CSIP_42.Model.Model.N = 11;
            CSIP_42.Model.Model.M = 10;
            CSIP_42.Model.Model.spawnSec = 0;

            CSIP_42.Model.Model model = new CSIP_42.Model.Model(true, "EntityConfigForSimulationTest.json");

            Guest guest = new Guest(model.EntityIdentifiers[TypeNameEnum.Guest], model.GetGameInfo().Table, model.GetMainGateGatePos());
            int happy = guest.Happiness;

            model.SelectForCurrentElement(TypeNameEnum.Road);
            List<Tuple<Position, int>> init = new List<Tuple<Position, int>>();
            init.Add(new Tuple<Position, int>(new Position(5, 5), 3));
            init.Add(new Tuple<Position, int>(new Position(6, 5), 4));
            init.Add(new Tuple<Position, int>(new Position(7, 5), 5));
            init.Add(new Tuple<Position, int>(new Position(8, 5), 6));
            init.Add(new Tuple<Position, int>(new Position(8, 4), 7));
            init.Add(new Tuple<Position, int>(new Position(8, 3), 8));
            init.Add(new Tuple<Position, int>(new Position(8, 2), 9));
            init.Add(new Tuple<Position, int>(new Position(7, 2), 10));
            init.Add(new Tuple<Position, int>(new Position(6, 2), 11));
            init.Add(new Tuple<Position, int>(new Position(5, 2), 12));
            init.Add(new Tuple<Position, int>(new Position(4, 2), 13));
            init.Add(new Tuple<Position, int>(new Position(3, 2), 14));

            for (int i = 0; i < init.Count; i++)
            {
                model.PlaceEntity(init[i].Item1, init[i].Item2);
            }

            model.SelectForCurrentElement(TypeNameEnum.HotDogRestaurant);
            model.SetEntryPriceForCurrentElement(0);
            model.PlaceEntity(new Position(1, 2), 15);

            model.SelectForCurrentElement(TypeNameEnum.GrassPlant);
            model.PlaceEntity(new Position(4,6), 16);
            model.SelectForCurrentElement(TypeNameEnum.FlowerPlant);
            model.PlaceEntity(new Position(7,6), 17);
            model.SelectForCurrentElement(TypeNameEnum.TreePlant);
            model.PlaceEntity(new Position(9,4), 18);

            (model.GetGameInfo().Table[8, 2] as Road).TrashCounter = 1;
            (model.GetGameInfo().Table[6, 2] as Road).TrashCounter = 3;

            GameObject gameinfo = model.GetGameInfo();
            gameinfo.People[TypeNameEnum.Guest].Add(guest);
            
            Dijkstra dijkstra = new Dijkstra(gameinfo.Graph, arc => 1, DijkstraMode.Sum);
            Node sourceNode = gameinfo.mainGate.Node;

            Node targetNode = (gameinfo.Table[1, 2] as Restaurant).Node;
            dijkstra.AddSource(sourceNode);
            dijkstra.RunUntilFixed(targetNode);
            var disst = dijkstra.GetDistance(targetNode);
            Path path = (dijkstra.GetPath(targetNode) as Path);
            guest.SetPath(path);

            Assert.AreEqual(happy, guest.Happiness);
            guest.Move();
            happy += 5;
            Assert.AreEqual(happy, guest.Happiness);           
            guest.Move();
            happy += 5;
            Assert.AreEqual(happy, guest.Happiness); 
            guest.Move();
            happy += 5;
            Assert.AreEqual(happy, guest.Happiness);
            guest.Move();
            happy += 30;
            Assert.AreEqual(happy, guest.Happiness);
            guest.Move();
            happy += 30;
            Assert.AreEqual(happy, guest.Happiness);
            guest.Move();
            happy += 40;
            Assert.AreEqual(happy, guest.Happiness);
            guest.Move();
            happy += 10;
            Assert.AreEqual(happy, guest.Happiness);
            guest.Move();
            happy += 10;
            happy -= guest.willingness * 10;
            Assert.AreEqual(happy, guest.Happiness);
            guest.Move();
            happy -= guest.willingness * 10;
            Assert.AreEqual(happy, guest.Happiness);
            guest.Move();
            happy -= guest.willingness * 40;
            Assert.AreEqual(happy, guest.Happiness);
            guest.Move();
            happy -= guest.willingness * 30;
            Assert.AreEqual(happy, guest.Happiness);
            guest.Move();
            happy -= guest.willingness * 30;
            Assert.AreEqual(happy, guest.Happiness);   
            guest.Move();
            Assert.AreEqual(happy, guest.Happiness);
            guest.Move();
            Assert.AreEqual(happy, guest.Happiness);

        }
        #endregion

        #region STAFF
        [TestMethod]
        public void Staff_walking()
        {
            CSIP_42.Model.Model.N = 11;
            CSIP_42.Model.Model.M = 10;
            CSIP_42.Model.Model.spawnSec = 0;

            CSIP_42.Model.Model model = new CSIP_42.Model.Model(true, "EntityConfigForSimulationTest.json");

            Cleaner cleaner = new Cleaner(model.EntityIdentifiers[TypeNameEnum.Cleaner], model.GetGameInfo().Table, model.GetMainGateGatePos());
            Mechanic meha = new Mechanic(model.EntityIdentifiers[TypeNameEnum.Mechanic], model.GetGameInfo().Table, model.GetMainGateGatePos());


            model.SelectForCurrentElement(TypeNameEnum.Road);
            List<Tuple<Position, int>> init = new List<Tuple<Position, int>>();
            init.Add(new Tuple<Position, int>(new Position(5, 5), 3));
            init.Add(new Tuple<Position, int>(new Position(6, 5), 4));
            init.Add(new Tuple<Position, int>(new Position(7, 5), 5));
            init.Add(new Tuple<Position, int>(new Position(8, 5), 6));
            init.Add(new Tuple<Position, int>(new Position(8, 4), 7));
            init.Add(new Tuple<Position, int>(new Position(8, 3), 8));
            init.Add(new Tuple<Position, int>(new Position(8, 2), 9));
            init.Add(new Tuple<Position, int>(new Position(7, 2), 10));
            init.Add(new Tuple<Position, int>(new Position(6, 2), 11));
            init.Add(new Tuple<Position, int>(new Position(5, 2), 12));
            init.Add(new Tuple<Position, int>(new Position(4, 2), 13));
            init.Add(new Tuple<Position, int>(new Position(3, 2), 14));

            for (int i = 0; i < init.Count; i++)
            {
                model.PlaceEntity(init[i].Item1, init[i].Item2);
            }

            GameObject gameinfo = model.GetGameInfo();
            
            Dijkstra dijkstra = new Dijkstra(gameinfo.Graph, arc => 1, DijkstraMode.Sum);
            Node sourceNode = gameinfo.mainGate.Node;
            Node targetNode = (gameinfo.Table[3, 2] as Road).Node;
            dijkstra.AddSource(sourceNode);
            dijkstra.RunUntilFixed(targetNode);
            var disst = dijkstra.GetDistance(targetNode);
            Path path = (dijkstra.GetPath(targetNode) as Path);
            cleaner.SetPath(path);
            meha.SetPath(path);

            Assert.IsFalse(cleaner.Move());
            Assert.IsFalse(meha.Move());
            Assert.AreEqual(cleaner.position,new Position(5,6));
            Assert.AreEqual(meha.position,new Position(5,6));

            for (int i = 0; i < init.Count-1; i++)
            {
                Assert.IsFalse(cleaner.Move());
                Assert.IsFalse(meha.Move());
                Assert.AreEqual(cleaner.position, init[i].Item1);
                Assert.AreEqual(meha.position, init[i].Item1);
            }
            Assert.IsTrue(cleaner.Move());
            Assert.IsTrue(meha.Move());
            Assert.AreEqual(cleaner.position, init[init.Count-1].Item1);
            Assert.AreEqual(meha.position, init[init.Count - 1].Item1);
        }

        [TestMethod]
        public void Cleaning() {
            CSIP_42.Model.Model.N = 11;
            CSIP_42.Model.Model.M = 10;
            CSIP_42.Model.Model.spawnSec = 0;

            CSIP_42.Model.Model model = new CSIP_42.Model.Model(true, "EntityConfigForSimulationTest.json");

            Cleaner cleaner = new Cleaner(model.EntityIdentifiers[TypeNameEnum.Cleaner], model.GetGameInfo().Table, model.GetMainGateGatePos());

            model.SelectForCurrentElement(TypeNameEnum.Road);
            List<Tuple<Position, int>> init = new List<Tuple<Position, int>>();
            init.Add(new Tuple<Position, int>(new Position(5, 5), 3));
            init.Add(new Tuple<Position, int>(new Position(6, 5), 4));
            init.Add(new Tuple<Position, int>(new Position(7, 5), 5));
            init.Add(new Tuple<Position, int>(new Position(8, 5), 6));
            init.Add(new Tuple<Position, int>(new Position(8, 4), 7));
            init.Add(new Tuple<Position, int>(new Position(8, 3), 8));
            init.Add(new Tuple<Position, int>(new Position(8, 2), 9));
            init.Add(new Tuple<Position, int>(new Position(7, 2), 10));
            init.Add(new Tuple<Position, int>(new Position(6, 2), 11));
            init.Add(new Tuple<Position, int>(new Position(5, 2), 12));
            init.Add(new Tuple<Position, int>(new Position(4, 2), 13));
            init.Add(new Tuple<Position, int>(new Position(3, 2), 14));

            var rand = new Random();
            (model.GetGameInfo().Table[5, 6] as Road).TrashCounter = rand.Next(1, 10); 
            for (int i = 0; i < init.Count; i++)
            {
                model.PlaceEntity(init[i].Item1, init[i].Item2);
                (model.GetGameInfo().Table[init[i].Item1.X, init[i].Item1.Y] as Road).TrashCounter = rand.Next(1,10);
            }
            

            GameObject gameinfo = model.GetGameInfo();

            Dijkstra dijkstra = new Dijkstra(gameinfo.Graph, arc => 1, DijkstraMode.Sum);
            Node sourceNode = gameinfo.mainGate.Node;

            Node targetNode = (gameinfo.Table[3, 2] as Road).Node;
            dijkstra.AddSource(sourceNode);
            dijkstra.RunUntilFixed(targetNode);
            var disst = dijkstra.GetDistance(targetNode);
            Path path = (dijkstra.GetPath(targetNode) as Path);
            cleaner.SetPath(path);
            cleaner.Move();
            cleaner.Move();
            Assert.AreEqual((model.GetGameInfo().Table[5, 6] as Road).TrashCounter, 0);
            for (int i = 0; i < init.Count; i++)
            {
                cleaner.Move();
                Assert.AreEqual((model.GetGameInfo().Table[init[i].Item1.X, init[i].Item1.Y] as Road).TrashCounter, 0);
            }
        }
        #endregion
    }
}
