﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Timers;
using Satsuma;
using CSIP_42.Model.Enumerators;


namespace CSIP_42.Model.Entity
{
    public class Guest : Person
    {
        public bool wannaThrowTrash;
        public int willingness;
        public bool wannaLeave;
        private int _happy;
        private int _hunger;
        private int _nature;
        private readonly Random rand;

        #region static
        public static int MinCash = 8000;
        public static int MaxCash = 35000;
        public static int MinHappy = 500;
        public static int MaxHappy = 1000;
        public static int MinHunger = 500;
        public static int MaxHunger = 1000;
        public static int MinNaturesCall = 500;
        public static int MaxNaturesCall = 1000;
        public static int MinWillingness = 1;
        public static int MaxWillingness = 3;     

        public static int HungerMaximumValue = 1000;
        public static int HappinessMaximumValue = 1000;
        public static int NaturesCallMaximumValue = 1000;
        public static int Sad_Okay_Limit = 300;
        public static int Okay_Happy_Limit = 700;

        public static int SecHungerModifier = 3;
        public static int SecHappyMinus = 3;
        public static int SecNaturesCallMinus = 3;
        #endregion

        #region PROPERTIES
        public int Happiness { get { return _happy; } set { _happy = value; if (_happy > HappinessMaximumValue ) { _happy = HappinessMaximumValue; } } }
        public int Hunger { get { return _hunger; } set { _hunger = value; if (_hunger > HungerMaximumValue) { _hunger = HungerMaximumValue; } } }
        public int NaturesCall { get { return _nature; } set { _nature = value; if (_nature > NaturesCallMaximumValue) { _nature = NaturesCallMaximumValue; } } }
        public int Cash { get; set; }
        public int ServizeTimeRemaining { get; set; }
        public int TrashTimeRemaining { get; set; }


        #endregion


        #region FUNCTIONS

        public Guest(EntityIdentifier identifier, Infrastructure[,] map, Position pos): base(map,pos, StatusEnum.Happy, identifier)
        {
            this.rand = new Random(DateTime.Now.Millisecond);
            Cash =          rand.Next(MinCash, MaxCash);
            Happiness =     rand.Next(MinHappy,MaxHappy);
            Hunger =        rand.Next(MinHunger,MaxHunger);
            NaturesCall =   rand.Next(MinNaturesCall,MaxNaturesCall);
            willingness =   rand.Next(MinWillingness,MaxWillingness);
            wannaThrowTrash = false;
            wannaLeave = false;
            TrashTimeRemaining = -1;
            StatusCheck();
        }

        /// <summary>
        /// step one if he is on the road
        /// decrease the level of happiness if he is waiting
        /// nothing if he is on a game|restaurant|toalett
        /// </summary>
        /// <returns>True if he arrived his taraget</returns>
        public override bool Move()
        {
            Happiness -= SecHappyMinus;
            Hunger -= SecHungerModifier;
            NaturesCall -= SecNaturesCallMinus;
            bool finish = false;
            switch (place)
            {
                case PersonPlaceEnum.OnRoad:
                    finish = Step();
                    break;
                case PersonPlaceEnum.InQueue:
                    Waiting();
                    break;
                case PersonPlaceEnum.InBuilding:
                    Enjoy();
                    break;
            }

            return finish;
        }

        private void Enjoy()
        {
            if(ServizeTimeRemaining > 0)
            {
                ServizeTimeRemaining--;
            }

        }

        private void Waiting()
        {
            Happiness -= 10;
            StatusCheck();
            return;
        }

        private bool Step()
        {
            Node currentNode;
            currentNode = (map[position.X, position.Y] as IGraphNodeID).Node;
            Node nextNode = pathToTarget.NextNode(currentNode);
            if (nextNode.Equals(Node.Invalid))
            {
                nextNode = pathToTarget.LastNode;
            }
            Position nextPos = position;
            int row = map.GetLength(0);
            int col = map.GetLength(1);


            if(TrashTimeRemaining >= 0)
            {
                TrashTimeRemaining--;
                wannaThrowTrash = false;
            }
            if(TrashTimeRemaining == 0)
            {
                wannaThrowTrash = true;
            }

            for (int i = -1; i <= 1; i++)
            {
                for (int j = -1; j <= 1; j++)
                {

                    if (0 > (position.X + i) ||
                        row <= (position.X + i) ||
                        0 > (position.Y + j) ||
                        col <= (position.Y + j) ||
                        map[position.X + i, position.Y + j] == null) 
                    { 
                        continue; 
                    }


                    switch (map[position.X + i, position.Y + j].Category)
                    {
                        case CategoryEnum.Bin:
                            if (TrashTimeRemaining >= 0)
                            {
                                TrashTimeRemaining = -1;
                                wannaThrowTrash = false;
                            }
                            break;
                        case CategoryEnum.MainGate:
                        case CategoryEnum.Game:
                        case CategoryEnum.Restaurant:
                        case CategoryEnum.Toalett:
                            if (nextNode == (map[position.X + i, position.Y + j] as IGraphNodeID).Node)
                            {
                                if(map[position.X + i, position.Y + j] is MainGate)
                                {
                                    nextPos = (map[position.X + i, position.Y + j] as MainGate).GatePosition;
                                }
                                else
                                {
                                    nextPos = (map[position.X + i, position.Y + j] as Building).GatePosition;
                                }
                            }
                            break;
                        case CategoryEnum.Road:
                            if ((map[position.X + i, position.Y + j] as Road).TrashCounter > 0)
                            {
                                Happiness -= willingness * 10 * (map[position.X + i, position.Y + j] as Road).TrashCounter;
                            }

                            if (nextNode == (map[position.X + i, position.Y + j] as IGraphNodeID).Node)
                            {
                                nextPos = map[position.X + i, position.Y + j].position;
                            }

                            break;
                        case CategoryEnum.Plant:
                            this.Happiness += (map[position.X + i, position.Y + j] as Plant).HappinessModifier;
                            break;

                        default:
                            break;
                    }
                }
            }
            StatusCheck();

            position = nextPos;
            if (pathToTarget.LastNode.Equals(nextNode)) {
                return true;
            }
            return false;

        }

        /// <summary>
        /// Set the actual status (feeling) of the guest
        /// </summary>
        public void StatusCheck()
        {
            if (Happiness <= 0 || Cash <= 0 || Hunger <= 0 || NaturesCall <= 0)
            {
                wannaLeave = true;
            }

            if ( Hunger < Sad_Okay_Limit || Happiness <  Sad_Okay_Limit || wannaLeave) { status = StatusEnum.Sad; }
            else if ( Hunger < Okay_Happy_Limit || Happiness < Okay_Happy_Limit) { status = StatusEnum.Okay; }
            else { status = StatusEnum.Happy; }

        }

        public override CategoryEnum WhatWant() {
            StatusCheck();

            if (wannaLeave) {
                return CategoryEnum.MainGate;
            }

            int HappyPiece = HappinessMaximumValue - Happiness;
            int HungerPiece = HungerMaximumValue - Hunger;
            int NaturePiece = NaturesCallMaximumValue - NaturesCall;
            
            if(HappyPiece + HungerPiece + NaturePiece == 0)
            {
                HappyPiece = HungerPiece = NaturePiece = 1;
            }
                

            int tmpRand = rand.Next(1,HappyPiece + HungerPiece + NaturePiece);
            if (tmpRand <= HungerPiece)
            {
                return CategoryEnum.Restaurant;
            }
            else if (tmpRand <= HappyPiece + HungerPiece)
            {
                return CategoryEnum.Game;
            }
            else
            {
                return CategoryEnum.Toalett;
            }
        }

        /// <summary>
        /// Choose a target from a list
        /// </summary>
        /// <param name="list">list of infrastructure elements</param>
        /// <returns>One concrete infrastructure what is okay to him</returns>
        public Infrastructure ChooseTarget(List<Infrastructure> list) {
            Infrastructure target = null;
            List<Infrastructure> tmpList = new List<Infrastructure>(list);
            bool thinking = true;
            double maxPrice = this.MaxPrice();

            while (tmpList.Count > 0 && thinking)
            {
                int index = rand.Next(tmpList.Count);
                if ((tmpList[index] as Building).ServicePrice < maxPrice)
                {
                    thinking = false;
                    target = tmpList[index];
                } else {
                    tmpList.RemoveAt(index);
                    if (tmpList.Count == 0) {
                        wannaLeave = true;
                        return null;
                    }
                }
            }
            return target;
        }

        /// <summary>
        /// Get the max price what (s)he allows to pay
        /// </summary>
        /// <returns>the price in double</returns>
        public double MaxPrice()
        {
            double maxPrice = 0;
             switch (willingness)
            {
                case 1:
                    maxPrice = Cash * 0.4;
                    if (maxPrice < 500) maxPrice = 500;
                    break;

                case 2:
                    maxPrice = Cash * 0.6;
                    if (maxPrice < 700) maxPrice = 700;
                    break;

                case 3:
                    maxPrice = Cash * 0.8;
                    if (maxPrice < 1000) maxPrice = 1000;
                    break;
                default:
                    maxPrice = Cash;
                    break;
            }

            if(maxPrice > Cash)
            {
                maxPrice = Cash;
            }
            return maxPrice;
        }
    }
}
#endregion