﻿using System;
using System.Collections.Generic;
using System.Text;
using Satsuma;
using CSIP_42.Model.Enumerators;

namespace CSIP_42.Model.Entity
{
    public class Mechanic : Staff
    {
        public Node node;
        public bool hasTarget;

        public Mechanic(EntityIdentifier identifier, Infrastructure[,] map=null, Position pos=null): base(map, pos, identifier)
        {
            hasTarget = false;
            if(map != null && pos != null)
                node = (map[pos.X, pos.Y] as IGraphNodeID).Node;
        }

        /// <summary>
        /// The person steps one if he is on the road, or does nothing if he is in a building
        /// </summary>
        /// <returns>True if he arrived his taraget</returns>
        public override bool Move()
        {
            if (place == PersonPlaceEnum.OnRoad)
            {
                Node currentNode;
                if ((map[position.X, position.Y] is IGraphNodeID))
                {
                    currentNode = (map[position.X, position.Y] as IGraphNodeID).Node;
                }
                else
                {
                    throw new Exception("Lost Person");
                }
                Node nextNode = pathToTarget.NextNode(currentNode);
                if (nextNode.Equals(Node.Invalid))
                {
                    nextNode = pathToTarget.LastNode;
                }
                Position nextPos = position;
                int row = map.GetLength(0);
                int col = map.GetLength(1);

                for (int i = -1; i <= 1; i++)
                {
                    for (int j = -1; j <= 1; j++)
                    {

                        if (0 > (position.X + i) ||
                            row <= (position.X + i) ||
                            0 > (position.Y + j) ||
                            col <= (position.Y + j) ||
                            map[position.X + i, position.Y + j] == null)
                        {
                            continue;
                        }

                        if (map[position.X + i, position.Y + j] is IGraphNodeID)
                        {
                            if (nextNode == (map[position.X + i, position.Y + j] as IGraphNodeID).Node)
                            {
                                if(map[position.X + i, position.Y + j] is Building)
                                {
                                    nextPos = (map[position.X + i, position.Y + j] as Building).GatePosition;
                                }
                                else
                                {
                                    nextPos = map[position.X + i, position.Y + j].position;
                                }
                            }
                        }
                    }
                }

                position = nextPos;
                node = (map[nextPos.X, nextPos.Y] as IGraphNodeID).Node;
                if (!node.Equals(nextNode))
                {
                    throw new Exception("ERROR Wrong Step!!");
                }
                if (nextNode == pathToTarget.LastNode)
                {
                    return true;
                }
                return false;
            }
            else
            {
                return false;
            }
        }
    }
    
}
