﻿using System;
using System.Collections.Generic;
using System.Text;
using CSIP_42.Model.Enumerators;

namespace CSIP_42.Model
{
    public class PlacingInformation
    {
        protected bool Equals(PlacingInformation other)
        {
            return _rotationTimes == other._rotationTimes && TicketPrice == other.TicketPrice && MinimumCapacity == other.MinimumCapacity;
        }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj)) return false;
            if (ReferenceEquals(this, obj)) return true;
            if (obj.GetType() != this.GetType()) return false;
            return Equals((PlacingInformation) obj);
        }

        public override int GetHashCode()
        {
            return HashCode.Combine(_rotationTimes, TicketPrice, MinimumCapacity);
        }

        public static bool operator ==(PlacingInformation left, PlacingInformation right)
        {
            return Equals(left, right);
        }

        public static bool operator !=(PlacingInformation left, PlacingInformation right)
        {
            return !Equals(left, right);
        }

        private int _rotationTimes;
        

        /// <summary>
        /// Returns rotation times normalised between 0,4
        /// </summary>
        public int RotationTimes { get { return _rotationTimes; } set { _rotationTimes = value % 4; } }
        public int TicketPrice { get;  set; }
        public int MinimumCapacity { get;  set; }


        public PlacingInformation(int ticketPrice=0, int minimumCapacity=0)
        {
            TicketPrice = ticketPrice;
            MinimumCapacity = minimumCapacity;
            _rotationTimes = 0;
        }

        public PlacingInformation(PlacingInformation other)
        {
            if (other == null)
            {
                other = new PlacingInformation();
            }
            TicketPrice = other.TicketPrice;
            MinimumCapacity = other.MinimumCapacity; 
            _rotationTimes = 0;

        }
    }

    public class DisplayInformation
    {
        protected bool Equals(DisplayInformation other)
        {
            return DisplayName == other.DisplayName;
        }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj)) return false;
            if (ReferenceEquals(this, obj)) return true;
            if (obj.GetType() != this.GetType()) return false;
            return Equals((DisplayInformation) obj);
        }

        public override int GetHashCode()
        {
            return (DisplayName != null ? DisplayName.GetHashCode() : 0);
        }

        public static bool operator ==(DisplayInformation left, DisplayInformation right)
        {
            return left is not null && right is not null && left.DisplayName == right.DisplayName;
        }

        public static bool operator !=(DisplayInformation left, DisplayInformation right)
        {
            return !Equals(left, right);
        }

        public DisplayInformation(string displayName = "-placeholder-")
        {
            DisplayName = displayName;
        }

        internal DisplayInformation(DisplayInformation other)
        {
            DisplayName = other.DisplayName;
        }

        public string DisplayName { get; set; }

    }

    public class EntityIdentifier
    {
        protected bool Equals(EntityIdentifier other)
        {
            return TimeToBuild == other.TimeToBuild && Capacity == other.Capacity && ServiceTime == other.ServiceTime && AlwaysCost == other.AlwaysCost && SingleCost == other.SingleCost && HappinessModifier == other.HappinessModifier && HungerModifier == other.HungerModifier && Placeable == other.Placeable && GeneratorRadius == other.GeneratorRadius && Type == other.Type && Category == other.Category && PriceToPlace == other.PriceToPlace && SizeM == other.SizeM && SizeN == other.SizeN && Equals(ConstructionInformation, other.ConstructionInformation) && Equals(GateVector, other.GateVector) && Equals(DisplayInformationEnglish, other.DisplayInformationEnglish);
        }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj)) return false;
            if (ReferenceEquals(this, obj)) return true;
            if (obj.GetType() != this.GetType()) return false;
            return Equals((EntityIdentifier) obj);
        }

        public override int GetHashCode()
        {
            var hashCode = new HashCode();
            hashCode.Add(TimeToBuild);
            hashCode.Add(Capacity);
            hashCode.Add(ServiceTime);
            hashCode.Add(AlwaysCost);
            hashCode.Add(SingleCost);
            hashCode.Add(HappinessModifier);
            hashCode.Add(HungerModifier);
            hashCode.Add(Placeable);
            hashCode.Add(GeneratorRadius);
            hashCode.Add((int) Type);
            hashCode.Add((int) Category);
            hashCode.Add(PriceToPlace);
            hashCode.Add(SizeM);
            hashCode.Add(SizeN);
            hashCode.Add(ConstructionInformation);
            hashCode.Add(GateVector);
            hashCode.Add(DisplayInformationEnglish);
            return hashCode.ToHashCode();
        }

        public static bool operator ==(EntityIdentifier left, EntityIdentifier right)
        {
            return Equals(left, right);
        }

        public static bool operator !=(EntityIdentifier left, EntityIdentifier right)
        {
            return !Equals(left, right);
        }

        public int TimeToBuild { get; set; }
        public int Capacity { get; set;  }
        public int ServiceTime { get; set; }
        public int AlwaysCost { get; set; }
        public int SingleCost { get; set; }
        public int HappinessModifier { get; set; }
        public int HungerModifier { get; set; }
        public bool Placeable { get; set; }
        public int GeneratorRadius { get; set; }
        public TypeNameEnum Type { get; set; }
        public CategoryEnum Category { get; set; }
        public int PriceToPlace { get; set; }
        public int SizeM { get; set; }
        public int SizeN { get; set; }
        public PlacingInformation ConstructionInformation { get; set; }
        public Position GateVector { get; set; }

        public DisplayInformation DisplayInformationEnglish { get;  set; }

        public EntityIdentifier()
        {

        }
        public EntityIdentifier(TypeNameEnum type, CategoryEnum category, int price, int sizeM, int sizeN)
        {
            this.Type = type;
            this.Category = category;
            this.PriceToPlace = price;
            this.SizeM = sizeM;
            this.SizeN = sizeN;
            this.ConstructionInformation = null;
        }

        public EntityIdentifier(EntityIdentifier other)
        {
            TimeToBuild = other.TimeToBuild;
            Capacity = other.Capacity;
            ServiceTime = other.ServiceTime;
            AlwaysCost = other.AlwaysCost;
            SingleCost = other.SingleCost;
            HappinessModifier = other.HappinessModifier;
            HungerModifier = other.HungerModifier;
            Placeable = other.Placeable;
            GeneratorRadius = other.GeneratorRadius;
            Type = other.Type;
            Category = other.Category;
            PriceToPlace = other.PriceToPlace;
            SizeM = other.SizeM;
            SizeN = other.SizeN;
            ConstructionInformation = new PlacingInformation(other.ConstructionInformation);
            GateVector = new Position(other.GateVector);
            DisplayInformationEnglish = new DisplayInformation(other.DisplayInformationEnglish);
        }

        public void SetTicketPriceUponConstruction(int price) 
        {
            if (ConstructionInformation == null)
            {
                ConstructionInformation = new PlacingInformation();
            }
            this.ConstructionInformation.TicketPrice = price;
        }

        public void SetMinimumCapacityUponConstruction(int capacity)
        {
            if (ConstructionInformation == null)
            {
                ConstructionInformation = new PlacingInformation();
            }
            this.ConstructionInformation.MinimumCapacity = capacity;
        }

      
    }
}
