using CSIP_42.Model.Entity;
using CSIP_42.Model.Entity.Plants;
using CSIP_42.Model.Entity.Restaurants;
using CSIP_42.Model.Entity.Games;
using CSIP_42.Model.Enumerators;
using Satsuma;
using System.Diagnostics;
using System;
using System.Collections.Generic;
using System.Timers;
using Newtonsoft.Json;

namespace CSIP_42.Model
{

    public class Model
    {

        #region static
        public static int spawnSec = 2;
        public static int N = 45;
        public static int M = 29;
        public static int MechanicWorkingPrice = 15;
        public static int ClosedGameGuestHappinessMinus = 20;
        public static int StartMoney = 1001000;
        #endregion

        #region Fields
        private GameObject gameInfo;
        private int spawnReminder;
        private Timer secTimer;
        private Satsuma.Dijkstra dijkstra;
        private Satsuma.Bfs bfs;
        private List<SimulationEventArgs> ListOfChangingElements;
        private List<Person> NewPersons;
        private List<Person> LeavingGuests;
        private List<Infrastructure> newInfra;
        private Dictionary<Building, Satsuma.Node> nodesOfBuildings;
        private bool replace;
        private long time;
        private readonly bool test;
        private static Random rand;
        private Queue<bool> ComeInGuestPercentQueue;
        #endregion

        #region Properties
        public EntityIdentifier CurrentID { get; set; }
        public Dictionary<TypeNameEnum, EntityIdentifier> EntityIdentifiers { get; private set; }
        

        #endregion

        #region JSON Parsing
        /// <summary>
        /// Parses the a configuration JSON string into a Dictionary of Types.
        /// </summary>
        /// <param name="configJson">A valid JSON String, containing all Entity config.</param>
        /// <returns>A Dictionary where keys are the Entity objects and values are their Identity</returns>
        private static Dictionary<TypeNameEnum, EntityIdentifier> ReadFromJSON(string configJson)
        {

            Dictionary<TypeNameEnum, EntityIdentifier> outputDictionary = new Dictionary<TypeNameEnum, EntityIdentifier>();

            Dictionary<string, EntityIdentifier> inputDictionary = JsonConvert.DeserializeObject<Dictionary<string, EntityIdentifier>>(configJson, new EntityParser());
            var input = JsonConvert.DeserializeObject(configJson);

            foreach (var item in inputDictionary)
            {
                if (Enum.TryParse(item.Key, out TypeNameEnum result))
                {
                    outputDictionary.Add(result, item.Value);
                }
                else
                {
                    throw new Exception("Cannot convert JSON key to TypeNameEnum:" + item.Key);
                }
            }


            return outputDictionary;
        }

        /// <summary>
        /// Reads a file from the Persistance poject
        /// </summary>
        /// <param name="fname"></param>
        /// <returns>The content in string.</returns>
        private static string ReadJSONConfigFromFile(string fname)
        {
            String configPath = Environment.CurrentDirectory;
            configPath = System.IO.Path.Combine(configPath, "Resources");
            configPath = System.IO.Path.Combine(configPath, fname);
            

            string configJson = System.IO.File.ReadAllText(configPath);
            return configJson;
        }
        #endregion

        #region Events
        public event EventHandler<List<SimulationEventArgs>> SimulationEvent;
        public event EventHandler<Tuple<long,int>> StatusBarChangedEvent;
        public event EventHandler<long> GameOverEvent;

        #endregion

        #region Constructor
        public Model(bool test = false, string jsonFilename = "PlaceableEntityConfigurations.json")
        {
            this.test = test;
            replace = false;
            string json = ReadJSONConfigFromFile(jsonFilename);
            EntityIdentifiers = ReadFromJSON(json);
            NewGame(!test);
        }
        #endregion

        #region Functions - Common & Basic
        /// <summary>
        /// Size of the game table.
        /// </summary>
        /// <returns>The size in a pair (n,m)</returns>
        public Tuple<int, int> GetSize()
        {
            return new Tuple<int, int>(gameInfo.SizeN, gameInfo.SizeM);
        }
        private void BuildStarterTable()
        {
            if (gameInfo == null || (secTimer != null && secTimer.Enabled))
            {
                return;
            }
            //Fkapu pitese
            SelectForCurrentElement(TypeNameEnum.MainGate);
            Position MainGatePost = new Position(gameInfo.SizeN / 2 - EntityIdentifiers[TypeNameEnum.MainGate].SizeN / 2, gameInfo.SizeM - EntityIdentifiers[TypeNameEnum.MainGate].SizeM);
            PlaceInfrastructure(MainGatePost, 1, false);
            gameInfo.mainGate = gameInfo.Table[MainGatePost.X, MainGatePost.Y] as MainGate;
            SelectForCurrentElement(TypeNameEnum.Road);
            PlaceInfrastructure(gameInfo.mainGate.GatePosition + new Position(0,-1), 2, false);
            
            //Kerits pitese
            //Jobb s baloldalt val pitkezs
            SelectForCurrentElement(TypeNameEnum.FenceBasic);
            for (int i = 1; i <= gameInfo.SizeN-2; i++)
            {
                Position position = new Position(i, 0);
                if (AreaEmpty(position))
                {
                    FenceBasic fence = new FenceBasic(position, EntityIdentifiers[CurrentID.Type]);
                    fence.Identity.ConstructionInformation.RotationTimes = 0;
                    if (position.X < gameInfo.SizeN && position.X >= 0 &&
                        position.Y < gameInfo.SizeM && position.Y >= 0)
                    {
                        gameInfo.Table[position.X, position.Y] = fence;
                        gameInfo.Infrastructures[CurrentID.Category].Add(fence);
                    }
                }
                
                position = new Position(i, gameInfo.SizeM-1);
                if (AreaEmpty(position))
                {
                    FenceBasic fence = new FenceBasic(position, EntityIdentifiers[CurrentID.Type]);
                    fence.Identity.ConstructionInformation.RotationTimes = 2;
                    if (position.X < gameInfo.SizeN && position.X >= 0 &&
                        position.Y < gameInfo.SizeM && position.Y >= 0)
                    {
                        gameInfo.Table[position.X, position.Y] = fence;
                        gameInfo.Infrastructures[CurrentID.Category].Add(fence);
                    }
                }
            }

            for (int i = 1; i <= gameInfo.SizeM-2; i++)
            {
                Position position = new Position(0, i);
                if (AreaEmpty(position))
                {
                    FenceBasic fence = new FenceBasic(position, EntityIdentifiers[CurrentID.Type]);
                    fence.Identity.ConstructionInformation.RotationTimes = 3;
                    if (position.X < gameInfo.SizeN && position.X >= 0 &&
                        position.Y < gameInfo.SizeM && position.Y >= 0)
                    {
                        gameInfo.Table[position.X, position.Y] = fence;
                        gameInfo.Infrastructures[CurrentID.Category].Add(fence);
                    }
                }

                position = new Position(gameInfo.SizeN - 1, i);
                if (AreaEmpty(position))
                {
                    FenceBasic fence = new FenceBasic(position, EntityIdentifiers[CurrentID.Type]);
                    fence.Identity.ConstructionInformation.RotationTimes = 1;
                    if (position.X < gameInfo.SizeN && position.X >= 0 &&
                        position.Y < gameInfo.SizeM && position.Y >= 0)
                    {
                        gameInfo.Table[position.X, position.Y] = fence;
                        gameInfo.Infrastructures[CurrentID.Category].Add(fence);
                    }
                }

            }
            SelectForCurrentElement(TypeNameEnum.FenceCorner);
            List<Position> Corner = new List<Position>();
            Corner.Add(new Position(gameInfo.SizeN - 1, 0));
            Corner.Add(new Position(gameInfo.SizeN - 1, gameInfo.SizeM - 1));
            Corner.Add(new Position(0, gameInfo.SizeM - 1));
            Corner.Add(new Position(0, 0));
            for (int i = 0; i < 4; i++)
            {
                Position position = Corner[i];
                if (AreaEmpty(position))
                {
                    FenceCorner fence = new FenceCorner(position, EntityIdentifiers[CurrentID.Type]);
                    fence.Identity.ConstructionInformation.RotationTimes = i;
                    if (position.X < gameInfo.SizeN && position.X >= 0 &&
                        position.Y < gameInfo.SizeM && position.Y >= 0)
                    {
                        gameInfo.Table[position.X, position.Y] = fence;
                        gameInfo.Infrastructures[CurrentID.Category].Add(fence);
                    }
                }
            }



        }

        /// <summary>
        /// Infrastructures in list.
        /// </summary>
        /// <returns>All infrastructures</returns>
        public List<Infrastructure> GetInfrastructures()
        {
            List<Infrastructure> list = new List<Infrastructure>();
            foreach (var infra in gameInfo.Infrastructures[CategoryEnum.MainGate])
            {
                list.Add(infra);
            }
            foreach (var infra in gameInfo.Infrastructures[CategoryEnum.Road])
            {
                list.Add(infra);
            }
            foreach (var infra in gameInfo.Infrastructures[CategoryEnum.Other])
            {
                list.Add(infra);
            }

            return list;
        }

        /// <summary>
        /// Returns the Game representation when Model is in testing mode
        /// </summary>
        /// <returns>GameObject if in Testing mode, null if not</returns>
        public GameObject GetGameInfo()
        {
            if (test)
                return gameInfo;
            return null;
        }

        /// <summary>
        /// Initiate a new game
        /// </summary>
        /// <param name="needTimer">If model should start the timer. Only modigy this, when you wish to test</param>
        public void NewGame(bool needTimer = true)
        {
            //Meretek beallitasa
            int sizeN = N;
            int sizeM = M;
            CurrentID = null;

            //Idozito elokeszitese
            if (secTimer != null)
            {
                secTimer.Stop();
            }


            //Jtkinformcis osztly ltrehozsa s a grf s szimulci elkszitese
            gameInfo = new GameObject(sizeN, sizeM, EntityIdentifiers[TypeNameEnum.MainGate],StartMoney);
            dijkstra = new Satsuma.Dijkstra(gameInfo.Graph, arc => 1, DijkstraMode.Sum);
            bfs = new Satsuma.Bfs(gameInfo.Graph);
            ListOfChangingElements = new List<SimulationEventArgs>();
            NewPersons = new List<Person>();
            LeavingGuests = new List<Person>();
            newInfra = new List<Infrastructure>(); 
            nodesOfBuildings = new Dictionary<Building, Node>();
            gameInfo.Open = false;
            spawnReminder = 0;
            time = 0;
            rand = new Random(DateTime.Now.Millisecond);
            ComeInGuestPercentQueue = new Queue<bool>();

            //Kezdeti plya megpitese (Kerites & Fkapu)
            BuildStarterTable(); 
            GetInfrastructures();
            //Szimulcis idzit elinditasa
            if(needTimer){
                secTimer = new Timer(1000);
                secTimer.Elapsed += SecTickedEventHandler;
                secTimer.Start();
            }
        }
        /// <summary>
        /// Start the game with the ticket price of the park
        /// </summary>
        /// <param name="ticketPrice">Entry ticket price for the park</param>
        public void StartGame(int ticketPrice)
        {
            gameInfo.Open = true;
            gameInfo.mainGate.TicketPrice = ticketPrice;
        }
        #endregion

        #region Functions - Building & Placing
        /// <summary>
        /// Lists the placeable entities
        /// </summary>
        /// <returns>The identity of placeables in List</returns>
        public List<EntityIdentifier> GetListOfPlaceable()
        {
            List<EntityIdentifier> output = new List<EntityIdentifier>();
            foreach (var item in EntityIdentifiers)
            {
                if (item.Value.Placeable)
                    output.Add(item.Value);
            }
            return output;
        }

        /// <summary>
        /// Request for Info for position
        /// </summary>
        /// <param name="pos">The object's position you want to request</param>
        /// <returns>Entity Info</returns>
        public EntityInfo GetEntitiyInfoForPosition(Position pos)
        {
            if (gameInfo.Table[pos.X, pos.Y] != null)
            {
                Infrastructure infra = gameInfo.Table[pos.X, pos.Y];

                if (infra is Game)
                {
                    Game game = infra as Game;
                    return new EntityInfo(
                        peopleWaiting: game.queue.Count,
                        peopleUsing: game.guests.Count,
                        minimumCapacity: game.MinCapacity,
                        maximumCapacity: game.Capacity,
                        ticketPrice: game.ServicePrice,
                        alwaysCost: game.AlwaysCost,
                        singleCost: game.SingleCost,
                        category: game.Category
                    ); 
                }

                else if (infra is Building) 
                {
                    Building building = infra as Building;
                    return new EntityInfo(
                        peopleWaiting: building.queue.Count,
                        peopleUsing: building.guests.Count,
                        minimumCapacity: 0,
                        maximumCapacity: building.Capacity,
                        ticketPrice: building.ServicePrice,
                        alwaysCost: building.AlwaysCost,
                        singleCost: building.SingleCost,
                        category: building.Category
                    );
                }
            }
            return null;
        }

        /// <summary>
        /// Selects the object for modification on position
        /// </summary>
        /// <param name="pos">Position of the object</param>
        /// <returns>The Category and the Type of the object, if given position was valid</returns>
        public Tuple<CategoryEnum,TypeNameEnum> SelectForModify(Position pos)
        {
            Infrastructure infra;
            try {
                infra = gameInfo.Table[pos.X, pos.Y];
            }
            catch (Exception)
            {
                throw new InvalidOperationException("Given position is not valid on the table!");
            }
            if (infra is Road)
            {
                Person cleaner = null;
                foreach (var person in gameInfo.People[TypeNameEnum.Cleaner])
                {
                    if (person.position == pos)
                    {
                        replace = true;
                        cleaner = person;
                    }
                }
                if (replace && cleaner != null)
                {
                    LeavingGuests.Add(cleaner);
                    return new Tuple<CategoryEnum, TypeNameEnum>(CategoryEnum.Staff,TypeNameEnum.Cleaner);

                }
                else
                {
                    throw new ArgumentException("There is no cleaner on the given location");
                }
            }

            else if (infra is Game || infra is Toalett || infra is Restaurant) 
            {
                CurrentID = infra.Identity;
            }

            else if (infra is MainGate)
            {
                CurrentID = infra.Identity;
            }

            else
            {
                throw new ArgumentException("This cannot be selected for modification!");
            }

            return new Tuple<CategoryEnum, TypeNameEnum>(infra.Category, infra.Type);

        }

        public Position GetMainGateGatePos()
        {
            return gameInfo.mainGate.GatePosition;
        }

        public Tuple<int,int> GetMainGateSize() 
        { 
            return new Tuple<int,int>(EntityIdentifiers[TypeNameEnum.MainGate].SizeM, EntityIdentifiers[TypeNameEnum.MainGate].SizeN);
        }

        /// <summary>
        /// Gets a sample Identifier for the Entity type
        /// </summary>
        /// <param name="type">Identity</param>
        /// <returns></returns>
        public EntityIdentifier GetEntityIdentifier(TypeNameEnum type)
        {
            return EntityIdentifiers[type];
        }

        /// <summary>
        /// Selects for the current element, before placing and rotation
        /// </summary>
        /// <param name="type">Entity type</param>
        public void SelectForCurrentElement(TypeNameEnum type)
        {
            CurrentID = new EntityIdentifier(GetEntityIdentifier(type));
        }
        public void SetEntryPriceForCurrentElement(int price)
        {
            CurrentID.SetTicketPriceUponConstruction(price);
        }
        public void SetMinCapacityForCurrentElement(int percent)
        {
            int minCap = Convert.ToInt32(Math.Round(CurrentID.Capacity * (percent / 100.0)));
            CurrentID.SetMinimumCapacityUponConstruction(minCap);
        }
        public int GetEntryPriceForCurrentElement()
        {
            return CurrentID.ConstructionInformation.TicketPrice;
        }
        public int GetMinCapacityForCurrentElement()
        {
            return Convert.ToInt32(Math.Round((CurrentID.ConstructionInformation.MinimumCapacity / (float)CurrentID.Capacity) * 100));
        }

        /// <summary>
        /// Checks if the current element can be placed to the desired position         
        /// </summary>
        /// <param name="pos">The desired position</param>
        /// <returns></returns>
        public bool AreaEmpty(Position pos)
        {

            if (CurrentID.Category == CategoryEnum.Staff)
            {
                if (CurrentID.SizeN != 1 || CurrentID.SizeM != 1)
                {
                    return false;
                }

                foreach (var item in gameInfo.People[TypeNameEnum.Cleaner])
                {
                    if (item.position == pos)
                    {
                        return false;
                    }
                }

                foreach (var item in gameInfo.People[TypeNameEnum.Mechanic])
                {
                    if (item.position == pos)
                    {
                        return false;
                    }
                }

                return (pos.X < gameInfo.SizeN && pos.X >= 0 &&
                        pos.Y < gameInfo.SizeM && pos.Y >= 0 &&
                        gameInfo.Table[pos.X, pos.Y] != null &&
                        gameInfo.Table[pos.X, pos.Y] is Road); 


            }
            else
            {
                for (int i = 0; i < CurrentID.SizeN; i++)
                {
                    for (int j = 0; j < CurrentID.SizeM; j++)
                    {
                        if (pos.X + i < gameInfo.SizeN && pos.X + i >= 0 &&
                            pos.Y + j < gameInfo.SizeM && pos.Y + j >= 0)
                        {
                            if (gameInfo.Table[pos.X + i, pos.Y + j] != null) return false;
                        }
                        else return false;
                    }
                }
            }



            return true;
        }

        /// <summary>
        /// Places the selected element to the desigered position.
        /// </summary>
        /// <param name="pos">Desired position</param>
        /// <param name="index">Easy acces index from VM</param>
        /// <returns></returns>
        public bool PlaceEntity(Position pos, int index)
        {
            if (CurrentID.Category == CategoryEnum.Staff)
            {
                return PlaceStaff(pos, CurrentID.Type);
            }
            else
            {
                return PlaceInfrastructure(pos, index);
            }
        }
        private bool PlaceInfrastructure(Position positionToPlace, int index, bool useTimer = true)
        {
            Entity.Entity currentElement = null;
            switch (CurrentID.Type)
            {
                case TypeNameEnum.Bin:
                    currentElement = new Bin(positionToPlace, CurrentID);
                    break;

                case TypeNameEnum.TreePlant:
                    currentElement = new TreePlant(positionToPlace, CurrentID);
                    break;

                case TypeNameEnum.FlowerPlant:
                    currentElement = new FlowerPlant(positionToPlace, CurrentID);
                    break;

                case TypeNameEnum.GrassPlant:
                    currentElement = new GrassPlant(positionToPlace, CurrentID);
                    break;

                case TypeNameEnum.Road:
                    currentElement = new Road(positionToPlace, StatusEnum.Operating, CurrentID);
                    break;

                case TypeNameEnum.Toalett:
                    currentElement = new Toalett(positionToPlace, CurrentID);
                    break;

                case TypeNameEnum.HotDogRestaurant:
                    currentElement = new HotDogRestaurant(positionToPlace, CurrentID);
                    break;

                case TypeNameEnum.IceCreamRestaurant:
                    currentElement = new IceCreamRestaurant(positionToPlace, CurrentID);
                    break;

                case TypeNameEnum.CottonCandyRestaurant:
                    currentElement = new CottonCandyRestaurant(positionToPlace, CurrentID);
                    break;

                case TypeNameEnum.SushiRestaurant:
                    currentElement = new SushiRestaurant(positionToPlace, CurrentID);
                    break;

                case TypeNameEnum.RollerCoasterGame:
                    currentElement = new RollerCoasterGame(positionToPlace, CurrentID);
                    break;

                case TypeNameEnum.GiantWheelGame:
                    currentElement = new GiantWheelGame(positionToPlace, CurrentID);
                    break;

                case TypeNameEnum.CarouselGame:
                    currentElement = new CarouselGame(positionToPlace, CurrentID);
                    break;

                case TypeNameEnum.MagicCastleGame:
                    currentElement = new MagicCastleGame(positionToPlace, CurrentID);
                    break;

                case TypeNameEnum.HorrorHouseGame:
                    currentElement = new HorrorHouseGame(positionToPlace, CurrentID);
                    break;

                case TypeNameEnum.DodgemGame:
                    currentElement = new DodgemGame(positionToPlace, CurrentID);
                    break;

                case TypeNameEnum.Generator:
                    currentElement = new Generator(positionToPlace, StatusEnum.Operating, CurrentID);
                    break;

                case TypeNameEnum.MainGate:
                    currentElement = new MainGate(positionToPlace, StatusEnum.Operating, CurrentID);
                    break;

                default:
                    currentElement = null;
                    break;
            }


            if (currentElement == null || !(currentElement is IPlaceable)) return false;
            Infrastructure infra = currentElement as Infrastructure;
            infra.index = index;

            if (gameInfo.Money - CurrentID.PriceToPlace < 0)
            {
                throw new InvalidOperationException("Not enough money!");
            }
            else
            {
                gameInfo.Money -= CurrentID.PriceToPlace;
                StatusBarChange();
            }

            if (AreaEmpty(positionToPlace))
            {
                infra.position = positionToPlace;
                for (int i = 0; i < CurrentID.SizeN; i++)
                {
                    for (int j = 0; j < CurrentID.SizeM; j++)
                    {
                        if (positionToPlace.X + i < gameInfo.SizeN && positionToPlace.X + i >= 0 &&
                            positionToPlace.Y + j < gameInfo.SizeM && positionToPlace.Y + j >= 0)
                        {
                            try
                            {
                                gameInfo.Table[positionToPlace.X + i, positionToPlace.Y + j] = infra;
                            }
                            catch (Exception)
                            {
                                return false;
                            }

                        }
                    }
                }
                //Grfba szrs megksrlse, amennyiben nem sikerl trjnk vissza hamissal
                try
                {
                    AddToGraph(positionToPlace, currentElement);
                }
                catch (Exception)
                {
                    return false;
                }

                try
                {
                    if (useTimer)
                    {
                        newInfra.Add(infra);
                    }
                    else
                    {
                        gameInfo.Infrastructures[CurrentID.Category].Add(infra);
                    }
                }
                catch (Exception)
                {
                    return false;
                }

                return true;
            }
            else
            {
                return false;
            }
        }
        private bool PlaceStaff(Position positionToPlace, TypeNameEnum staffType)
        {
            Person StaffToPlace = null;
            switch (staffType)
            {
                case TypeNameEnum.Cleaner:
                    StaffToPlace = new Cleaner(EntityIdentifiers[staffType], gameInfo.Table, positionToPlace);
                    break;
                case TypeNameEnum.Mechanic:
                    StaffToPlace = new Mechanic(EntityIdentifiers[staffType], gameInfo.Table, positionToPlace);
                    break;
                default:
                    throw new InvalidOperationException("You cannot place this as a placeable person");
            }

            if (!replace)
            {
                if (gameInfo.Money - CurrentID.PriceToPlace < 0)
                {
                    throw new InvalidOperationException("Not enough money!");
                }
                else
                {
                    gameInfo.Money -= CurrentID.PriceToPlace;
                    StatusBarChange();
                }
            }
            else 
            {
                replace = false;
            }

            try
            {
                NewPersons.Add(StaffToPlace);
            }
            catch (Exception)
            {
                return false;
            }
            return true;
        }
        private void AddToGraph(Position pos, Entity.Entity currentElement)
        {

            if (currentElement is IGraphNodeID && currentElement is Entity.Entity)
            {
                if ((currentElement as Entity.Entity).position != pos)
                {
                    throw new InvalidOperationException("Given position shoudl be equal to the current element's position");
                }
                IGraphNodeID currentElementWithGraphNode = currentElement as IGraphNodeID;
                currentElementWithGraphNode.Node = gameInfo.Graph.AddNode();

                Position entryPositionForCurrentElement = currentElement.position;
                if (currentElement is IHaveGate)
                {
                    entryPositionForCurrentElement = (currentElement as IHaveGate).GatePosition;
                }

                // Csak a falszomszdos terleti elemeket kell megvizsglnunk, hogy grfbeli elemek vagy sem,
                // hiszen a sarok csatolakozst nem vesszk csatlakozsnak.
                //  -,- | 0,- | +,-
                //  -,0 | x,y | +,0
                //  -,+ | 0,+ | +,+
                // Ciklussal iterlunk, ha i,j ciklus vltozk szorzata nem nulla, akkor vagy sarok elem vagy az aktulis elem,
                // amit pedig a kett egyidejleg nullra val tesztelssel ki lehet zrni

                for (int i = -1; i <= 1; i++)
                {
                    for (int j = -1; j <= 1; j++)
                    {
                        if (i * j != 0 || (i == 0 && j == 0)) continue;


                        if (entryPositionForCurrentElement.X + i < gameInfo.SizeN && entryPositionForCurrentElement.X + i >= 0 &&
                            entryPositionForCurrentElement.Y + j < gameInfo.SizeM && entryPositionForCurrentElement.Y + j >= 0)
                        {
                            Entity.Infrastructure infra = null;
                            try
                            {
                                infra = gameInfo.Table[entryPositionForCurrentElement.X + i, entryPositionForCurrentElement.Y + j];
                            }
                            catch (IndexOutOfRangeException)
                            {
                                continue;
                            }
                            if (infra == null) continue;

                            if (infra is IGraphNodeID)
                            {

                                if ((infra is IHaveGate && (infra as IHaveGate).GatePosition == new Position(entryPositionForCurrentElement.X + i, entryPositionForCurrentElement.Y + j)) ||
                                    (!(infra is IHaveGate)))
                                {
                                    Arc a = gameInfo.Graph.AddArc(currentElementWithGraphNode.Node, (infra as IGraphNodeID).Node, Directedness.Undirected);
                                    
                                }
                            }
                        }

                    }
                }
                if(currentElement is Building)
                {
                    nodesOfBuildings.Add(currentElement as Building, currentElementWithGraphNode.Node);
                }
            }
        }
        /// <summary>
        /// Rotates current element
        /// </summary>
        /// <returns>Thruple: (the new size of element by N, the new size of element by M, the rotation times) </returns>
        public Tuple<int,int,int> RotateCurrentElement()
        {

            CurrentID.ConstructionInformation.RotationTimes++;

            CurrentID.GateVector = new Position((-1) * CurrentID.GateVector.Y + CurrentID.SizeM -1, CurrentID.GateVector.X);

            int tmp = CurrentID.SizeN;
            CurrentID.SizeN = CurrentID.SizeM;
            CurrentID.SizeM = tmp;

            return new Tuple<int,int,int>(CurrentID.SizeN, CurrentID.SizeM, CurrentID.ConstructionInformation.RotationTimes);
        }

        #endregion

        #region Functions - Simulation

        private void Simulation() {
            if (SimulationEvent != null)
            {
                List<SimulationEventArgs> tmpList = new List<SimulationEventArgs>(ListOfChangingElements);
                ListOfChangingElements.Clear();
                this.SimulationEvent?.Invoke(this, tmpList);
            }
        }
        private void StatusBarChange()
        {
            if(StatusBarChangedEvent != null)
            {
                int percent = -1;
                if (ComeInGuestPercentQueue.Count == 10)
                {
                    percent = 0;
                    foreach (bool yes in ComeInGuestPercentQueue)
                    {
                        if (yes)
                        {
                            percent += 10;
                        }
                    }
                }

                this.StatusBarChangedEvent?.Invoke(this,new Tuple<long,int>(GetMoney(), percent));
            }
        }

        private void GameOver()
        {
            secTimer.Stop();
            if(GameOverEvent != null)
            {
                this.GameOverEvent?.Invoke(this, time);
            }
        }
        public long GetMoney()
        {
            return gameInfo.Money;
        }


        /// <summary>
        /// create and place a new guest on the park in every "spawnSec" secound
        /// </summary>
        private void SpawnGuest() 
        {
            spawnReminder++;
            if (spawnReminder == spawnSec)
            {
                bool In = false;
                Guest guest = new Guest(EntityIdentifiers[TypeNameEnum.Guest], gameInfo.Table, gameInfo.mainGate.GatePosition); 
                if (guest.MaxPrice() >= gameInfo.mainGate.TicketPrice)
                {
                    
                    guest.Cash -= gameInfo.mainGate.TicketPrice;
                    gameInfo.People[TypeNameEnum.Guest].Add(guest);
                    PersonNeedANewTarget(guest);
                    if (guest.wannaLeave)
                    {
                        In = false;
                        guest.Cash += gameInfo.mainGate.TicketPrice;
                    }
                    else
                    {
                        In = true;
                        gameInfo.Money += gameInfo.mainGate.TicketPrice;
                    }
                }
                spawnReminder = 0;

                if (ComeInGuestPercentQueue.Count == 10)
                {
                    ComeInGuestPercentQueue.Dequeue();
                    ComeInGuestPercentQueue.Enqueue(In);
                }
                else
                {
                    ComeInGuestPercentQueue.Enqueue(In);
                }
            }
        }

        /// <summary>
        /// every people on the road step one
        /// </summary>
        private void PeopleStep()
        {
            foreach (Person person in gameInfo.People[TypeNameEnum.Guest])
            {
                Position prevPos = person.position;
                if (person.Move())
                {
                    if (person.place == PersonPlaceEnum.OnRoad)
                    {
                        PersonArrived(person);
                    }
                }
                AddToChangingList(person, prevPos, person.position);
            }

            foreach (Person person in gameInfo.People[TypeNameEnum.Mechanic])
            {
                Position prevPos = person.position;
                if (person.Move())
                {
                    if (person.place == PersonPlaceEnum.OnRoad)
                    {
                        PersonArrived(person);
                    }
                }
                AddToChangingList(person, prevPos, person.position);
                gameInfo.Money -= (person as Staff).AlwaysCost;
            }


            foreach (Person person in gameInfo.People[TypeNameEnum.Cleaner])
            {
                Position prevPos = person.position;
                if (person.Move())
                {
                    if (person.place == PersonPlaceEnum.OnRoad)
                    {
                        PersonArrived(person);
                    }
                }
                if (!LeavingGuests.Contains(person))
                {
                    AddToChangingList(person, prevPos, person.position);
                }
                gameInfo.Money -= (person as Staff).AlwaysCost;
            }
        }

        /// <summary>
        /// Arranging things per second for games
        /// </summary>
        private void GameSec()
        {
            foreach (Game game in gameInfo.Infrastructures[CategoryEnum.Game])
            {
               gameInfo.Money -= game.AlwaysCost;

                if (game.hasElectricity && game.status == StatusEnum.Waiting)
                {
                    while (game.guests.Count < game.Capacity && game.queue.Count > 0)
                    {
                        game.guests.Enqueue(game.queue.Dequeue());
                    }
                    if (game.guests.Count >= game.MinCapacity)
                    {
                        foreach (Guest guest in game.guests)
                        {
                            guest.place = PersonPlaceEnum.InBuilding;
                            guest.ServizeTimeRemaining = game.ServiceTime;
                            guest.Cash -= game.ServicePrice;
                            gameInfo.Money += game.ServicePrice;
                        }
                        gameInfo.Money -= game.SingleCost;
                        game.serviceTimeRemaining = game.ServiceTime;
                        game.status = StatusEnum.Operating;
                        AddToChangingList(game);
                    }
                }
                if (game.status == StatusEnum.Operating)
                {
                     game.serviceTimeRemaining--;
                    if (game.serviceTimeRemaining == 0)
                    {

                        game.usageLevel -= game.guests.Count;
                        
                        foreach (Guest guest in game.guests)
                        {
                            guest.Happiness += game.HappinessModifier;
                            guest.place = PersonPlaceEnum.OnRoad;
                            PersonNeedANewTarget(guest);
                        }
                        game.guests.Clear();


                        if (game.usageLevel <= 0)
                        {
                            BuildingBreakDown(game);
                            AddToChangingList(game);
                        }
                        else
                        {
                            game.status = StatusEnum.Waiting;
                            AddToChangingList(game);
                        }

                    }
                }
                if (game.status == StatusEnum.Repair)
                {
                    game.repairTimeRemaining--;
                    gameInfo.Money -= MechanicWorkingPrice;
                    if (game.repairTimeRemaining == 0)
                    {
                        game.usageLevel = rand.Next(50, 100);
                        game.status = StatusEnum.Waiting;
                        game.mechanic.place = PersonPlaceEnum.OnRoad;
                        game.mechanic.hasTarget = false;
                        Mechanic tmpMec = game.mechanic;
                        game.mechanic = null;
                        PersonNeedANewTarget(tmpMec);
                        AddToChangingList(game);
                    }
                }
            }

        }
        /// <summary>
        /// Arranging things per second for restaurants
        /// </summary>
        private void RestaurantSec()
        {
            foreach (Restaurant restaurant in gameInfo.Infrastructures[CategoryEnum.Restaurant])
            {             
                gameInfo.Money -= restaurant.AlwaysCost;

                while (restaurant.guests.Count > 0 && restaurant.guests.Peek().ServizeTimeRemaining <= 0)
                {
                    Guest guest = restaurant.guests.Dequeue();
                    guest.place = PersonPlaceEnum.OnRoad;
                    guest.Hunger += restaurant.HungerModifier;
                    guest.NaturesCall -= restaurant.HungerModifier / 3;
                    guest.TrashTimeRemaining = rand.Next(1,restaurant.ServiceTime+1);
                    PersonNeedANewTarget(guest);
                }


                while (restaurant.guests.Count < restaurant.Capacity && restaurant.queue.Count > 0)
                {
                    Guest guest = restaurant.queue.Dequeue();
                    guest.Cash -= restaurant.ServicePrice;
                    gameInfo.Money += restaurant.ServicePrice;
                    guest.place = PersonPlaceEnum.InBuilding;
                    guest.ServizeTimeRemaining = restaurant.ServiceTime;
                    restaurant.guests.Enqueue(guest);
                    gameInfo.Money -= restaurant.SingleCost;
                }
            }

        }
        /// <summary>
        /// Arranging things per second for toalets
        /// </summary>
        private void ToalettSec()
        {
            foreach (Toalett toalett in gameInfo.Infrastructures[CategoryEnum.Toalett])
            {
                
                gameInfo.Money -= toalett.AlwaysCost;
                while (toalett.guests.Count > 0 && toalett.guests.Peek().ServizeTimeRemaining <= 0)
                {
                    Guest guest = toalett.guests.Dequeue();
                    guest.NaturesCall = rand.Next(Convert.ToInt32(Guest.NaturesCallMaximumValue * 0.7), Guest.NaturesCallMaximumValue);
                    guest.place = PersonPlaceEnum.OnRoad;
                    PersonNeedANewTarget(guest);
                }


                while (toalett.guests.Count < toalett.Capacity && toalett.queue.Count > 0)
                {
                    Guest guest = toalett.queue.Dequeue();
                    guest.Cash -= toalett.ServicePrice;
                    gameInfo.Money += toalett.ServicePrice;
                    guest.place = PersonPlaceEnum.InBuilding;
                    guest.ServizeTimeRemaining = toalett.ServiceTime;
                    toalett.guests.Enqueue(guest);
                    gameInfo.Money -= toalett.SingleCost;
                }
            }
        }
        /// <summary>
        /// Arranging things per second for decorations
        /// </summary>
        private void DecorationSec()
        {
            foreach (Plant plant in gameInfo.Infrastructures[CategoryEnum.Plant])
            {
                gameInfo.Money -= plant.AlwaysCost;
            }
            foreach(Bin bin in gameInfo.Infrastructures[CategoryEnum.Bin])
            {
                gameInfo.Money -= bin.AlwaysCost;
            }
        }

        /// <summary>
        /// Find and set a new Target and a path to a person
        /// </summary>
        /// <param name="person">the person who needs a new target and path</param>
        private void PersonNeedANewTarget(Person person) 
        {
            person.place = PersonPlaceEnum.OnRoad;
            Infrastructure Target = null;
            Position senderPos = person.position;
            Node node;
            if (gameInfo.Table[senderPos.X, senderPos.Y] is IGraphNodeID)
            {
                node = (gameInfo.Table[senderPos.X, senderPos.Y] as IGraphNodeID).Node;
            }
            else { throw new Exception("Lost Person"); }


            CategoryEnum category = person.WhatWant();
            if (category == CategoryEnum.MainGate && person is Guest)
            {
                SetPersonPath(node, gameInfo.mainGate.Node, person);
                return;
            }

            List<Infrastructure> inf = SearchGoodNode(node, category);
            if (person is Guest)
            {
                Target = (person as Guest).ChooseTarget(inf);
                if (Target == null)
                {
                    (person as Guest).wannaLeave = true;
                    SetPersonPath(node, gameInfo.mainGate.Node, person);
                    return;
                }

            }
            else if (person is Cleaner)
            {
                Target = inf[rand.Next(inf.Count - 1)];
            }
            else if (person is Mechanic) 
            {
                List<Infrastructure> games = SearchGoodNode(node,CategoryEnum.Game).FindAll(e => (e.status == StatusEnum.Broken && (e as Game).mechanic == null));
                bfs = new Satsuma.Bfs(gameInfo.Graph);
                bfs.AddSource(node);
                bfs.Run();
                int level = -1;
                Game ChosenOne = null;
                foreach (Game game in games)
                {
                    if (ChosenOne == null)
                    {
                        if (game.mechanic == null && bfs.Reached(game.Node))
                        {
                            ChosenOne = game;
                            level = bfs.GetLevel(game.Node);
                        }
                    }
                    else if (game.mechanic == null && bfs.Reached(game.Node) && level > bfs.GetLevel(game.Node))
                    {
                        ChosenOne = game;
                        level = bfs.GetLevel(game.Node);
                    }
                }
                if (ChosenOne == null) {
                    Target = inf[rand.Next(inf.Count - 1)];
                }
                else
                {
                    Target = ChosenOne;
                    (person as Mechanic).hasTarget = true;
                    ChosenOne.mechanic = person as Mechanic;
                }
            }

            else
            { throw new Exception("unknown people in the park"); }

            SetPersonPath(node, (Target as IGraphNodeID).Node, person);
        }
        /// <summary>
        /// Somebody arrived his target it will handles.
        /// If a Guest arrived a building, then let inside and waiting or enjoying that
        /// If a Guest arrived the main gate, then (s)he left the park
        /// If a Staff arrived a road, then set a new target for him/her
        /// If a Mechanic arrived his broken game, then let inside to repair it
        /// </summary>
        /// <param name="person">the person who arrived</param>
        private void PersonArrived(Person person) 
        {
            Infrastructure inf = gameInfo.Table[person.position.X, person.position.Y];

            if (!(inf is IGraphNodeID))
            {
                throw new Exception("Lost Person");
            }

            if (person is Guest)
            {
                if (inf is MainGate)
                {
                    PersonLeave(person);
                    return;
                }
                else if (inf is Building)
                {
                    
                    if ((inf is Game && (!(inf as Game).hasElectricity || inf.status == StatusEnum.Broken || inf.status == StatusEnum.Repair)) || (inf as Building).ServicePrice > (person as Guest).MaxPrice())
                    {
                        (person as Guest).Happiness -= ClosedGameGuestHappinessMinus; 
                        PersonNeedANewTarget(person);
                        return;
                    }
                    else
                    {
                        (inf as Building).queue.Enqueue(person as Guest);
                        person.place = PersonPlaceEnum.InQueue;
                        return;
                    }
                }
                else
                {
                    PersonNeedANewTarget(person);
                    return;
                }

            }
            else if (person is Staff)
            {
                if (person is Mechanic && inf is Game && inf.status == StatusEnum.Broken)
                {
                    (inf as Game).mechanic = (person as Mechanic);
                    (inf as Game).repairTimeRemaining = rand.Next((100 - (inf as Game).usageLevel) / 5, 100 - (inf as Game).usageLevel);
                    (inf as Game).status = StatusEnum.Repair;
                    AddToChangingList(inf);
                    person.place = PersonPlaceEnum.InBuilding;
                    return;

                }
                else
                {
                    PersonNeedANewTarget(person);
                    return;
                }
            }
            else
            {
                throw new Exception("unknown people in the park!");
            }

        }
        private void PersonLeave(Person person)
        {
            if (person is Guest)
            {
                LeavingGuests.Add(person);
            }
        }
        /// <summary>
        /// Because of the foreach loop,
        /// we can't modify the list content in tha main time so we do it later in this section
        /// </summary>
        private void PeopleListsToDictionary()
        {
            if (NewPersons.Count > 0)
            {
                List<Person> tmpNewPersons = new List<Person>(NewPersons);
                NewPersons.Clear();
                foreach (Person person in tmpNewPersons)
                {
                    gameInfo.People[person.Type].Add(person);
                    PersonNeedANewTarget(person);
                }
            }
            if (LeavingGuests.Count > 0)
            {
                List<Person> tmpLeavingGuests = new List<Person>(LeavingGuests);
                LeavingGuests.Clear();
                foreach (Person person in tmpLeavingGuests)
                {
                    gameInfo.People[person.Type].Remove(person);
                }
            }
        }


        private double ArcWeight(Satsuma.Arc arc, Satsuma.Node sourceNode ,Satsuma.Node targetNode, double w = double.PositiveInfinity)
        {
            var nodes = new List<Satsuma.Node>(nodesOfBuildings.Values);
            return (nodes.Contains(gameInfo.Graph.U(arc)) || nodes.Contains(gameInfo.Graph.V(arc))) && (!gameInfo.Graph.V(arc).Equals(targetNode) && !gameInfo.Graph.U(arc).Equals(targetNode)) && (!gameInfo.Graph.V(arc).Equals(sourceNode) && !gameInfo.Graph.U(arc).Equals(sourceNode)) ? w : 1;

        }
        private void SetPersonPath(Node sourceNode, Node targetNode, Person person)
        {
            if (person is Guest && !targetNode.Equals(gameInfo.mainGate.Node))
            {
                dijkstra = new Satsuma.Dijkstra(gameInfo.Graph, arc => ArcWeight(arc, sourceNode, targetNode), DijkstraMode.Sum);
            }
            else if (person is Cleaner) {
                dijkstra = new Satsuma.Dijkstra(gameInfo.Graph, arc => ArcWeight(arc, sourceNode, targetNode,100), DijkstraMode.Sum);
            }
            else{
                dijkstra = new Satsuma.Dijkstra(gameInfo.Graph, arc => 1, DijkstraMode.Sum);
            }

            dijkstra.AddSource(sourceNode);
            dijkstra.RunUntilFixed(targetNode);
            var disst = dijkstra.GetDistance(targetNode);
            Path path = (dijkstra.GetPath(targetNode) as Path);
            if(path == null )
            {
                SetPersonPath(sourceNode, gameInfo.mainGate.Node, person);
                if(person is Guest)
                {
                    (person as Guest).wannaLeave = true;
                }
            }
            else
            {
                person.SetPath(path);
            }
        }
        private List<Infrastructure> SearchGoodNode(Node sourceNode, CategoryEnum cat)
        { /** CSAK olyat szabad tadni categorynak aminek van trolt node-ja*/
            bfs = new Satsuma.Bfs(gameInfo.Graph);
            bfs.AddSource(sourceNode);
            bfs.Run();
            List<Node> subgraph = new List<Node>(bfs.ReachedNodes);
            List<Infrastructure> infras = gameInfo.Infrastructures[cat].FindAll(x => x is IGraphNodeID);
            return infras.FindAll(x => subgraph.Contains((x as IGraphNodeID).Node));

        }
        private void BuildingBreakDown(Game game) 
        {
            foreach (Guest guest in game.queue)
            {
                guest.Happiness -= ClosedGameGuestHappinessMinus;
                PersonNeedANewTarget(guest);
            }
            game.queue.Clear();
            game.status = StatusEnum.Broken;
            bfs = new Satsuma.Bfs(gameInfo.Graph);
            bfs.AddSource(game.Node);
            bfs.Run();
            int level=-1;
            Mechanic ChosenOne = null;
            foreach (Mechanic meha in gameInfo.People[TypeNameEnum.Mechanic])
            {
                if(ChosenOne == null) 
                {
                    if (!meha.hasTarget && bfs.Reached(meha.node))
                    {
                        ChosenOne = meha;
                        level = bfs.GetLevel(meha.node);
                    }
                }
                else if (!meha.hasTarget && bfs.Reached(meha.node) && level > bfs.GetLevel(meha.node))
                {
                    ChosenOne = meha;
                    level = bfs.GetLevel(meha.node);
                }
            }
            if(ChosenOne != null)
            {
                ChosenOne.hasTarget = true;
                SetPersonPath(ChosenOne.node, game.Node, ChosenOne);
            }
            game.mechanic = ChosenOne;
        }
        private void AddToChangingList(Infrastructure infra)
        {
            if (infra is Building)
            {
                if(infra is Game)
                {
                ListOfChangingElements.Add(new SimulationEventArgs(infra.index, infra.Type, infra.status, true, (infra as Building).queue.Count, (infra as Building).guests.Count, (infra as Game).MinCapacity));
                }
                else
                {
                ListOfChangingElements.Add(new SimulationEventArgs(infra.index, infra.Type, infra.status, true, (infra as Building).queue.Count, (infra as Building).guests.Count));
                }
            }
            else
            {
                ListOfChangingElements.Add(new SimulationEventArgs(infra.index, infra.Type, infra.status));
            }
        }
        private void AddToChangingList(Person person, Position prevPos, Position nextPos)
        {
            if(gameInfo.Table[prevPos.X, prevPos.Y] is Road)
            {
                int index1 = gameInfo.Table[prevPos.X, prevPos.Y].index;
                StatusEnum status = StatusEnum.Clean;
                if (person is Cleaner)
                {
                    status = StatusEnum.Clean;
                }
                else
                {
                    if (person is Guest)
                    {
                        (gameInfo.Table[prevPos.X, prevPos.Y] as Road).TrashCounter += ((person as Guest).wannaThrowTrash ? 1 : 0);
                        (person as Guest).wannaThrowTrash = false;
                    }

                    if ((gameInfo.Table[prevPos.X, prevPos.Y] as Road).TrashCounter == 0)
                    {
                        status = StatusEnum.Clean;
                    }
                    else if ((gameInfo.Table[prevPos.X, prevPos.Y] as Road).TrashCounter == 1)
                    {
                        status = StatusEnum.Dirty;
                    }
                    else
                    {
                        status = StatusEnum.VeryDirty;
                    }
                }
                ListOfChangingElements.Insert(0, new SimulationEventArgs(index1, TypeNameEnum.Road, status));
            }
            if(gameInfo.Table[nextPos.X, nextPos.Y] is Road)
            {
                int index2 = gameInfo.Table[nextPos.X, nextPos.Y].index;
                ListOfChangingElements.Add(new SimulationEventArgs(index2, person.Type, person.status));
            }
        }
        /// <summary>
        /// Imitates a Tick when model is in testing mode
        /// </summary>
        public void TestTick()
        {
            if (test) SecTickedEventHandler(this, new EventArgs());
        }

        private void Build()
        {
            int index = 0;
            while (index < newInfra.Count)
            {
                if (newInfra[index] is Building)
                {
                    (newInfra[index] as Building).TimeToBuild -= 1;
                    if ((newInfra[index] as Building).TimeToBuild <= 0)
                    {
                        if (newInfra[index] is Game)
                        {
                            newInfra[index].status = StatusEnum.Waiting;
                        }
                        else
                        {
                            newInfra[index].status = StatusEnum.Operating;
                        }
                        AddToChangingList(newInfra[index] as Building);
                        gameInfo.Infrastructures[newInfra[index].Category].Add(newInfra[index]);
                        newInfra.RemoveAt(index);
                    }
                    else
                    {
                        index++;
                    }
                }
                else
                {
                    gameInfo.Infrastructures[newInfra[index].Category].Add(newInfra[index]);
                    newInfra.RemoveAt(index);
                }
            }
        }
        #endregion


        #region Event handlers
        private void SecTickedEventHandler(Object sender, EventArgs e) 
        {
            time++;
            Build();
            if (gameInfo.Open)
            {
                SpawnGuest(); 
                PeopleStep(); 
                GameSec();
                RestaurantSec();
                ToalettSec();
                DecorationSec();
            }
            StatusBarChange();
            if(gameInfo.Money < 0)
            {
                GameOver();
            }
            PeopleListsToDictionary();
            Simulation();
        }


        #endregion
    }
}
