﻿using System;

namespace CSIP_42.Model
{
    /// <summary>
    /// An Euclidian Point in a 2 dimensional space
    /// </summary>
    public class Position
    {
        /// <summary>
        /// Constructs a Point, to the given coordinates
        /// </summary>
        /// <param name="x">X axis</param>
        /// <param name="y">Y axis</param>
        public Position(int x, int y)
        {
            X = x;
            Y = y;
        }
        /// <summary>
        /// Copies another point.
        /// </summary>
        /// <param name="other">Point to copy</param>
        public Position(Position other)
        {
            X = other.X;
            Y = other.Y;
        }
        /// <summary>
        /// Creates a default constructed point to the origin: (0,0)
        /// </summary>
        public Position() : this(0, 0) { }
        public int X { get; set; }
        public int Y { get; set; }

        public override bool Equals(object obj)
        {
            return obj is Position position &&
                   X == position.X &&
                   Y == position.Y;
        }

        public override int GetHashCode()
        {
            return HashCode.Combine(X, Y);
        }

        public static Boolean operator ==(Position a, Position b)
        {
            return a.Equals(b);
        }

        public static Boolean operator !=(Position a, Position b)
        {
            return !(a == b);
        }

        public static Position operator +(Position A, Position B)
        {
            return new Position(A.X + B.X, A.Y + B.Y); //Eltolásként értelmezve
        }

        public static Position operator -(Position A, Position B)
        {
            return new Position(A.X - B.X, A.Y - B.Y); //Eltolásként értelmezve
        }

    }
}
