﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using CSIP_42.Model;
using CSIP_42.View;
using CSIP_42.ViewModel;
using Gat.Controls;


namespace CSIP_42
{
    /// <summary>
    /// Interaction logic for App.xaml
    /// </summary>
    public partial class App : Application
    {
        Model.Model _model;
        MainWindow _mainWindow;
        MainViewModel _mainViewModel;
        InfoWindow _infoWindow;
        GameSettingsDialogWindow _gameSettingsDialogWindow;
        ParkSettingsDialogWindow _parkSettingsDialogWindow;
        ServiceSettingsDialogWindow _serviceSettingsDialogWindow;
       
        public App()
        {
            Startup += new StartupEventHandler(App_Startup);
        }

        private void App_Startup(object sender, StartupEventArgs e)
        {
            _model = new Model.Model();

            _mainViewModel = new MainViewModel(_model);
            _mainViewModel.OpenInfoWindowEvent += new EventHandler(OpenInfoWindowEventHandler);
            _mainViewModel.OpenGameSettingsDialogWindowEvent += new EventHandler(OpenGameSettingsDialogWindowEventHandler);
            _mainViewModel.OpenServiceSettingsDialogWindowEvent += new EventHandler(OpenServiceSettingsDialogWindowEventHandler);
            _mainViewModel.OpenParkSettingsDialogWindowEvent += new EventHandler(OpenParkSettingsDialogWindowEventHandler);
            _mainViewModel.CloseParkSettingsDialogWindowEvent += new EventHandler(CloseParkSettingsDialogWindowEventHandler);
            _mainViewModel.CloseGameSettingsDialogWindowEvent += new EventHandler(CloseGameSettingsDialogWindowEventHandler);
            _mainViewModel.CloseServiceSettingsDialogWindowEvent += new EventHandler(CloseServiceSettingsDialogWindowEventHandler);
            _mainViewModel.ErrorEvent += new EventHandler<string>(ErrorEventHandler);
            _mainViewModel.GameOverEvent += new EventHandler<long>(GameOverEventHandler);
            _mainViewModel.ExitGame += new EventHandler(ExitGameEventHandler);

            _mainWindow = new MainWindow
            {
                // mindegyik windowhoz ugyanaz a vm tartozik, az infowindowhoz nem kell, az csak statikus
                DataContext = _mainViewModel
            };

            _mainWindow.Show();         
        }

      
        #region EventHandlers

        private void OpenInfoWindowEventHandler(object sender, EventArgs e)
        {
            _infoWindow = new InfoWindow();
            _infoWindow.ShowDialog();   
        }

        private void OpenParkSettingsDialogWindowEventHandler(object sender, EventArgs e)
        {
            _parkSettingsDialogWindow = new ParkSettingsDialogWindow
            {
                DataContext = _mainViewModel
            };
            _parkSettingsDialogWindow.ShowDialog();
        }

        private void OpenServiceSettingsDialogWindowEventHandler(object sender, EventArgs e)
        {
            _serviceSettingsDialogWindow = new ServiceSettingsDialogWindow
            {
                DataContext = _mainViewModel
            };
            _serviceSettingsDialogWindow.ShowDialog();
        }

        private void OpenGameSettingsDialogWindowEventHandler(object sender, EventArgs e)
        {
            _gameSettingsDialogWindow = new GameSettingsDialogWindow
            {
                DataContext = _mainViewModel
            };
            _gameSettingsDialogWindow.ShowDialog();
        }

        private void CloseServiceSettingsDialogWindowEventHandler(object sender, EventArgs e)
        {
            _serviceSettingsDialogWindow.Close();
        }

        private void CloseParkSettingsDialogWindowEventHandler(object sender, EventArgs e)
        {
            _parkSettingsDialogWindow.Close();
        }

        private void CloseGameSettingsDialogWindowEventHandler(object sender, EventArgs e)
        {
            _gameSettingsDialogWindow.Close();
        }

        private void ErrorEventHandler(object sender, string e)
        {
            MessageBox.Show(e,"Invalid input");
        }

        private void GameOverEventHandler(object sender, long e)
        {
            _mainWindow.Dispatcher.Invoke(() =>
            {
                // This messagebox allows to costumize the icon
                Gat.Controls.MessageBoxView messageBox = new Gat.Controls.MessageBoxView();
                Gat.Controls.MessageBoxViewModel vm = (Gat.Controls.MessageBoxViewModel)messageBox.FindResource("ViewModel");

                TimeSpan t = TimeSpan.FromSeconds(e);
                vm.Message = "You lost! You have led your business into debt in "
                    + string.Format("{0:D2}h:{1:D2}m:{2:D2}s", t.Hours, t.Minutes, t.Seconds) + ".\n"
                    + "But don't be sad life always gives you another chance...\n" +
                    "Do you want to start a new game now?";
                vm.Yes = "Yes";
                vm.No = "No";
                vm.OkVisibility = false;
                vm.YesVisibility = true;
                vm.NoVisibility = true;
                vm.Image = new BitmapImage(new System.Uri(@"/Images/GameOver.png", UriKind.Relative));
                vm.Caption = "CSIP-42 Theme Park";
                vm.Position = MessageBoxPosition.CenterOwner;
                vm.Owner = MainWindow;
                if (vm.Show() == Gat.Controls.MessageBoxResult.Yes)
                    _mainViewModel.NewGameCommand.Execute(null);
                else
                    _mainWindow.Close();
            });
        }

        private void ExitGameEventHandler(object sender, EventArgs e)
        {
            if (MessageBox.Show("Are you sure you want to exit?", "CSIP-42 Theme Park", 
                System.Windows.MessageBoxButton.YesNo, System.Windows.MessageBoxImage.Question) == System.Windows.MessageBoxResult.Yes)
                _mainWindow.Dispatcher.Invoke(() => _mainWindow.Close());
        }

        #endregion
    }
}
